@@ -4,11 +4,14 @@
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
 import logging
-import numpy as np
 import os
 
+import numpy as np
+
 from .passes import mil_passes
+from ..backend_helper import _get_colorspace_enum, _validate_image_input_output_shapes
 from coremltools import _SPECIFICATION_VERSION_IOS_15
+from coremltools import _OPSET
 from coremltools.converters.mil.backend.mil.helper import (
     cast_to_framework_io_dtype,
     create_file_value,
@@ -25,16 +28,15 @@ from coremltools.converters.mil.mil import (
     types
 )
 from coremltools.converters.mil.backend.nn.load import _set_optional_inputs
-from coremltools.converters.mil.input_types import ImageType, TensorType, EnumeratedShapes, RangeDim
+from coremltools.converters.mil.input_types import ColorLayout, ImageType, TensorType, EnumeratedShapes, RangeDim
 from coremltools.converters.mil.mil.ops.registry import SSAOpRegistry
 from coremltools.converters.mil.mil.types.symbolic import (
     any_symbolic,
     any_variadic,
     is_symbolic,
 )
-from coremltools.converters.mil.mil.types.type_mapping import types_int64
 from coremltools.libmilstoragepython import _BlobStorageWriter as BlobWriter
-from coremltools.models.model import _WEIGHTS_FILE_NAME
+from coremltools.models.utils import _WEIGHTS_FILE_NAME
 from coremltools.models.neural_network.flexible_shape_utils import (
     add_enumerated_image_sizes,
     add_multiarray_ndshape_enumeration,
@@ -55,9 +57,10 @@ def should_use_weight_file(val):
         val is not None
         and isinstance(val, (np.ndarray, np.generic))
         and val.size >= 10
-        and val.dtype in ['float16', 'float32']
+        and val.dtype in ['float16', 'float32', 'uint8', 'int8']
     )
 
+
 def translate_const(op, blob_writer):
     output_var = op.outputs[0]
 
@@ -77,6 +80,32 @@ def translate_const(op, blob_writer):
     )
 
 
+def translate_constexpr(op, blob_writer):
+
+    def get_value(var):
+        if should_use_weight_file(var.val):
+            value = create_file_value(var, blob_writer)
+        else:
+            value = create_immediate_value(var)
+
+        return value
+
+    output_var = op.outputs[0]
+
+    attributes = {"name": create_scalar_value(op.name)}
+    attributes.update({k: get_value(v) for k, v in op.inputs.items()})
+
+    return pm.Operation(
+        type=op.op_type,
+        attributes=attributes,
+        outputs=[
+            pm.NamedValueType(
+                name=output_var.name, type=types_to_proto(output_var.sym_type)
+            )
+        ],
+    )
+
+
 def translate_generic_op(op, parameters, blob_writer, literal_params=[]):
     inputs = {}
     for param_name, vars in op.inputs.items():
@@ -105,8 +134,7 @@ def translate_generic_op(op, parameters, blob_writer, literal_params=[]):
     ]
     blocks = None
     if len(op.blocks) > 0:
-        blocks = [create_block(b, parameters, blob_writer) \
-                  for b in op.blocks]
+        blocks = [create_block(b, parameters, blob_writer) for b in op.blocks]
 
     op_type = op.op_type
     attr_dict = {}
@@ -133,8 +161,13 @@ def translate_generic_op(op, parameters, blob_writer, literal_params=[]):
         outputs=outputs,
     )
 
-
 def create_block(block, parameters, blob_writer):
+
+    def feeds_to_only_constexprs(op):
+        return (op.op_type == 'const') \
+               and len(op.outputs[0].child_ops) > 0 \
+               and all((child_op.op_type.startswith("constexpr_")) for child_op in op.outputs[0].child_ops)
+
     proto_ops = []
 
     # Find the const op that generates classify's "label" / "class" string vec.
@@ -152,14 +185,24 @@ def create_block(block, parameters, blob_writer):
     for op in block.operations:
         op_cls_name = type(op).__name__
         if op_cls_name == "const":
+            if feeds_to_only_constexprs(op):
+                continue
             # Do not serialize the const op that creates the var bound to the classifier's "classes" param.
             # The variable's value will be bound directly to classify's "classes" param instead.
             if op != classify_const_classes_op:
                 proto_ops.append(translate_const(op, blob_writer))
+        elif op_cls_name.startswith("constexpr_"):
+            proto_ops.append(translate_constexpr(op, blob_writer))
         elif op_cls_name == "classify":
             # Classify's "classes" param should be serialized as a value literal bound
             # directly to the param, rather than as a const-generated variable.
             proto_ops.append(translate_generic_op(op, parameters, blob_writer, ["classes"]))
+        elif op_cls_name == "reshape_like":
+            # The reshape_like should also be able to take value from a const op
+            # This is a workaround solution
+            # rdar://98689808 (Reshape_like should also accept const value from non literal input)
+            literal_params = ["begins", "ends", "end_masks"]
+            proto_ops.append(translate_generic_op(op, parameters, blob_writer, literal_params))
         else:
             proto_ops.append(translate_generic_op(op, parameters, blob_writer))
 
@@ -174,7 +217,7 @@ def create_block(block, parameters, blob_writer):
     return pm.Block(inputs=inputs, outputs=output_names, operations=proto_ops)
 
 
-def convert_function(function, parameters, blob_writer):
+def convert_function(function, parameters, blob_writer, opset):
     block = create_block(function, parameters, blob_writer)
 
     inputs = []
@@ -182,8 +225,7 @@ def convert_function(function, parameters, blob_writer):
         proto_type = types_to_proto(var.sym_type)
         inputs.append(pm.NamedValueType(name=name, type=proto_type))
 
-    return pm.Function(inputs=inputs, opset="CoreML5", block_specializations={"CoreML5": block})
-
+    return pm.Function(inputs=inputs, opset=opset, block_specializations={opset: block})
 
 # Add a classify op to the output.
 # Replaces the original probabilites output (in the containing MIL block)
@@ -193,6 +235,12 @@ def _add_classify_op(prog, classifier_config):
     '''
     Add a "classify" op to the program, at the end of the main block
     '''
+    def remove_output(block, prob_var):
+        for i in range(len(block.outputs)):
+            if block.outputs[i] is prob_var:
+                block.outputs.pop(i)
+                break
+
     block = prog.functions["main"]
 
     message = "Class labels must be a list of integers / strings or a file path"
@@ -219,7 +267,12 @@ def _add_classify_op(prog, classifier_config):
         if isinstance(classes[0], int):
             classes = [np.int64(x) for x in classes]
         classes_var = mb.const(val=mil_list(classes))
-        out = mb.classify(probabilities=probability_var, classes=classes_var)
+        if probability_var.dtype != types.fp32:
+            remove_output(block, probability_var)
+            probability_var = mb.cast(x=probability_var, dtype="fp32", name=probability_var.name + "_cast_to_fp32")
+        out = mb.classify(probabilities=probability_var,
+                          classes=classes_var
+                          )
 
         predicted_feature_name = "classLabel" if classifier_config.predicted_feature_name is None \
                                               else classifier_config.predicted_feature_name
@@ -227,14 +280,11 @@ def _add_classify_op(prog, classifier_config):
         out[1].name = predicted_feature_name + "_probs"
 
         # Remove probabilities from block outputs, replace with classify's outputs
-        for i in range(0, len(block.outputs)):
-            if block.outputs[i] is probability_var:
-                block.outputs.pop(i)
-                break
+        remove_output(block, probability_var)
         block.outputs[:0] = out
         return out[0].name, out[1].name
 
-def load(prog, weights_dir, resume_on_errors=False, **kwargs):
+def load(prog, weights_dir, resume_on_errors=False, specification_version=_SPECIFICATION_VERSION_IOS_15, **kwargs):
     if "main" not in prog.functions:
         raise ValueError("main function not found in program")
 
@@ -248,19 +298,32 @@ def load(prog, weights_dir, resume_on_errors=False, **kwargs):
         predicted_feature_name, predicted_probabilities_name = _add_classify_op(prog, classifier_config)
 
     input_types = prog.main_input_types
+    output_types = prog.main_output_types
     weight_path = os.path.join(weights_dir, _WEIGHTS_FILE_NAME)
     blob_writer = BlobWriter(weight_path)
 
+    opset = _OPSET[specification_version]
+
     function_protos = {}
     for func_name, func in prog.functions.items():
-        function_protos[func_name] = convert_function(func, prog.parameters,
-            blob_writer)
+        function_protos[func_name] = convert_function(func, prog.parameters, blob_writer, opset)
 
     proto = pm.Program(
         version=1,
         functions=function_protos,
     )
 
+    desc = kwargs.get("model_description", None)
+    if desc and not isinstance(desc, ml.ModelDescription):
+        raise ValueError("Invalid model descriptor")
+
+    if desc:
+        if classifier_config is not None:
+            raise AssertionError("Both model_description and classifier_config can't be provided")
+        model = ml.Model(description=desc, specificationVersion=specification_version)
+        model.mlProgram.CopyFrom(proto)
+        return model
+
     input_features = []
     output_features = []
     symbolic_inputs = []
@@ -306,22 +369,13 @@ def load(prog, weights_dir, resume_on_errors=False, **kwargs):
                 array_type = ft.ArrayFeatureType(shape=shape, dataType=cast_to_framework_io_dtype(var, False))
                 input_feature_type.multiArrayType.CopyFrom(array_type)
             else:
-                if len(shape) < 3:
-                    raise ValueError("Image input, '{}', must have rank at least 3. Instead it has rank {}".
-                                     format(name, len(shape)))
                 # make a feature type of Type "imageType"
                 input_type = image_input_names[name]
+                _validate_image_input_output_shapes(input_type.color_layout, shape, name, is_input=True)
                 if not input_type.channel_first:
                     raise ValueError("Image input, '{}', must be in the channel_first format".
                                      format(name))
-
-                if input_type.color_layout == "G":
-                    clr_space = ft.ImageFeatureType.ColorSpace.GRAYSCALE
-                elif input_type.color_layout == "BGR":
-                    clr_space = ft.ImageFeatureType.ColorSpace.BGR
-                else:
-                    clr_space = ft.ImageFeatureType.ColorSpace.RGB
-
+                clr_space = _get_colorspace_enum(input_type.color_layout)
                 image_type = ft.ImageFeatureType(width=shape[-1],
                                                  height=shape[-2],
                                                  colorSpace=clr_space)
@@ -337,20 +391,44 @@ def load(prog, weights_dir, resume_on_errors=False, **kwargs):
         else:
             raise NotImplementedError()
 
-    for var in prog.functions["main"].outputs:
+    if output_types is not None and classifier_config is None:
+        assert len(output_types) == len(prog.functions["main"].outputs), \
+                "number of mil program outputs do not match the number of outputs provided by the user"
+
+    for i, var in enumerate(prog.functions["main"].outputs):
         output_feature_type = ft.FeatureType()
         if types.is_tensor(var.sym_type) or types.is_primitive(var.sym_type):
-            dataType = None
-            if classifier_config is None or var.name != predicted_feature_name:
-                # Not a classifier output, make sure model output type matches with ML Program type.
-                dataType = cast_to_framework_io_dtype(var, True)
+            if output_types is not None and isinstance(output_types[i], ImageType):
+                if not types.is_tensor(var.sym_type):
+                    raise ValueError("Image output, '{}', is a scalar, but it should be a tensor of rank 4".format(
+                                      var.name))
+                shape = var.sym_type.get_shape()
+                if any_variadic(shape):
+                    raise ValueError("Variable rank model outputs, that are ImageTypes, are not supported")
+                if any([is_symbolic(d) for d in shape]):
+                    raise NotImplementedError("Image output '{}' has symbolic dimensions in its shape".
+                                              format(var.name))
+                _validate_image_input_output_shapes(output_types[i].color_layout, shape, var.name, is_input=False)
+                clr_space = _get_colorspace_enum(output_types[i].color_layout)
+                image_type = ft.ImageFeatureType(width=shape[-1],
+                                                 height=shape[-2],
+                                                 colorSpace=clr_space)
+                output_feature_type.imageType.CopyFrom(image_type)
+                output_features.append(
+                    ml.FeatureDescription(name=var.name, type=output_feature_type)
+                )
             else:
-                # Classifier outputs are set up separately, so default to fp32 for now.
-                dataType = ft.ArrayFeatureType.ArrayDataType.FLOAT32
+                dataType = None
+                if classifier_config is None or var.name != predicted_feature_name:
+                    # Not a classifier output, make sure model output type matches with ML Program type.
+                    dataType = cast_to_framework_io_dtype(var, True)
+                else:
+                    # Classifier outputs are set up separately, so default to fp32 for now.
+                    dataType = ft.ArrayFeatureType.ArrayDataType.FLOAT32
 
-            array_type = ft.ArrayFeatureType(shape=None, dataType=dataType)
-            output_feature_type.multiArrayType.CopyFrom(array_type)
-            output_features.append(ml.FeatureDescription(name=var.name, type=output_feature_type))
+                array_type = ft.ArrayFeatureType(shape=None, dataType=dataType)
+                output_feature_type.multiArrayType.CopyFrom(array_type)
+                output_features.append(ml.FeatureDescription(name=var.name, type=output_feature_type))
         elif (types.is_dict(var.sym_type)):
             output_feature_type.dictionaryType.MergeFromString(b"")
             keytype, valtype = var.sym_type.T
@@ -381,7 +459,7 @@ def load(prog, weights_dir, resume_on_errors=False, **kwargs):
                 break
 
     # Create ML Model
-    model = ml.Model(description=desc, specificationVersion=_SPECIFICATION_VERSION_IOS_15)
+    model = ml.Model(description=desc, specificationVersion=specification_version)
     model.mlProgram.CopyFrom(proto)
 
     # Set symbolic shapes
