@@ -3,17 +3,18 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-import pytest
-import itertools
-import numpy as np
 import copy
+import itertools
+
+import numpy as np
+import pytest
 
 # import mil internal ops to add it to the builder
 import coremltools as ct
-from coremltools.converters.mil.mil import Builder as mb
+from coremltools.converters.mil.mil import Builder as mb, types
 from coremltools.converters.mil.mil.passes.pass_registry import PASS_REGISTRY
-from coremltools.converters.mil import types
 from coremltools.converters.mil.mil.types import string_to_builtin, builtin_to_string, promote_types
+from coremltools.converters.mil._deployment_compatibility import AvailableTarget as target
 
 # Set the testing backend
 import coremltools.converters.mil.testing_reqs as testing_reqs
@@ -148,6 +149,87 @@ class TestAdjustToSupportedTypes:
         assert prev_inputs[0][1].name == inputs[0][1].name
         assert inputs[0][1].dtype == types.fp32
 
+
+    @pytest.mark.parametrize(
+        "opset_version",
+        [None, target.iOS13, target.iOS16],
+    )
+    def test_float16_input_output(self, opset_version):
+        """
+        Input graph:
+
+        main(%x: (1, 1, 1, 1, fp16)(Tensor)) {
+            block0() {
+                %relu_0: (1, 1, 1, 1, fp16)(Tensor) = relu(x=%x, name="relu_0")
+            } -> (%relu_0)
+        }
+
+        Output graph (if opset_version < ios16):
+
+        main(%x: (1, 1, 1, 1, fp32)(Tensor)) {
+            block0() {
+                %cast_0: (1, 1, 1, 1, fp16)(Tensor) = cast(x=%x, dtype="fp16", name="cast_0")
+                %relu_0__pre__output__fp32__cast: (1, 1, 1, 1, fp16)(Tensor) = relu(x=%cast_0, name="relu_0")
+                %relu_0: (1, 1, 1, 1, fp32)(Tensor) = cast(x=%relu_0__pre__output__fp32__cast, dtype="fp32", name="cast_1")
+            } -> (%relu_0)
+        }
+
+        Output graph (if opset_version >= ios16): same as the input graph
+        """
+        @mb.program(input_specs=[mb.TensorSpec(shape=(1, 1, 1, 1), dtype=types.fp16)], opset_version=opset_version)
+        def prog(x):
+            return mb.relu(x=x)
+
+        prev_prog, prev_block, block = apply_pass_and_basic_check(
+            prog, "mil_backend::adjust_io_to_supported_types"
+        )
+
+        prev_inputs = list(prev_block.inputs.items())
+        inputs = list(block.inputs.items())
+        prev_outputs = prev_block.outputs
+        outputs = block.outputs
+        assert prev_inputs[0][1].name == inputs[0][1].name
+        assert outputs[0].name == prev_outputs[0].name
+        if opset_version is None or opset_version < target.iOS16:
+            assert get_op_types_in_program(prog) == ['cast', 'relu', 'cast']
+            assert inputs[0][1].dtype == types.fp32
+            assert outputs[0].dtype == types.fp32
+        else:
+            assert get_op_types_in_program(prog) == ['relu']
+            assert inputs[0][1].dtype == types.fp16
+            assert block.outputs[0].dtype == types.fp16
+            
+    def test_float16_input_output_with_opset_version_inference(self):
+        """
+        Input graph:
+
+        main(%x: (1, 1, 4, 4, fp16)(Tensor)) {
+          block0() {
+            %pixel_unshuffle_0: (1, 4, 2, 2, fp16)(Tensor) = pixel_unshuffle(x=%x, downscale_factor=2, name="pixel_unshuffle_0")
+          } -> (%pixel_unshuffle_0)
+        }
+        
+        This function would be inferred as an iOS16 function, and the graph pass should behave properly
+        """
+        @mb.program(input_specs=[mb.TensorSpec(shape=(1, 1, 4, 4), dtype=types.fp16)])
+        def prog(x):
+            x = mb.pixel_unshuffle(x=x, downscale_factor=np.uint32(2))
+            return x
+            
+        prev_prog, prev_block, block = apply_pass_and_basic_check(
+            prog, "mil_backend::adjust_io_to_supported_types"
+        )
+
+        prev_inputs = list(prev_block.inputs.items())
+        inputs = list(block.inputs.items())
+        prev_outputs = prev_block.outputs
+        outputs = block.outputs
+        assert prev_inputs[0][1].name == inputs[0][1].name
+        assert outputs[0].name == prev_outputs[0].name
+        assert get_op_types_in_program(prog) == ['pixel_unshuffle']
+        assert inputs[0][1].dtype == types.fp16
+        assert block.outputs[0].dtype == types.fp16
+
     def test_int8_input(self):
         """
         Input graph:
@@ -203,6 +285,7 @@ class TestAdjustToSupportedTypes:
             );
         } -> (out_0, out_1)
         """
+        pytest.xfail("fp64 dtype not supported in MIL")
         def body(a, b):
             return mb.add(x=a, y=np.float64(1)), b
 
@@ -251,6 +334,7 @@ class TestAdjustToSupportedTypes:
             fp32 y = cast(x=x, dtype="fp32")
         } -> (y)
         """
+        pytest.xfail("cast operation does not support casting to fp64")
         @mb.program(input_specs=[mb.TensorSpec(shape=(1, 1, 1, 1), dtype=types.int32)])
         def prog(x):
             y = mb.cast(x=x, dtype="fp64")
@@ -285,6 +369,7 @@ class TestAdjustToSupportedTypes:
         func main(int32 x) {
         } -> (x)
         """
+        pytest.xfail("cast not supports dtype=`int64`")
         @mb.program(input_specs=[mb.TensorSpec(shape=(1, 1, 1, 1), dtype=types.int32)])
         def prog(x):
             y = mb.cast(x=x, dtype="int64")
@@ -405,10 +490,10 @@ class TestImagePreprocessingPass:
             return z
 
         prog.main_input_types = (ct.ImageType(name='x',
-                                             shape=[1, 1, 20, 20],
-                                             bias=2.0,
-                                             color_layout="G",
-                                             channel_first=True),)
+                                              shape=[1, 1, 20, 20],
+                                              bias=2.0,
+                                              color_layout="G",
+                                              channel_first=True),)
 
         prev_prog, prev_block, block = apply_pass_and_basic_check(
             prog, "mil_backend::insert_image_preprocessing_ops"
@@ -715,6 +800,58 @@ class TestSanitizerPass:
         assert block.find_ops(op_type="relu")[1].name == "op_1"
         assert block.find_ops(op_type="add")[0].name == "op_3"
 
+    def test_sanitize_var_names_with_two_functions(self):
+        """
+        Input:
+            main(%x: (1, 3, 20, fp32)(Tensor)) {
+              block0() {
+                %var_1!: (1, 3, 20, fp32)(Tensor) = relu(x=%x, name="var_1!")
+              } -> (%var_1!)
+            }
+
+            main_2(%x: (1, 3, 20, fp32)(Tensor)) {
+              block0() {
+                %var_1!: (1, 3, 20, fp32)(Tensor) = relu(x=%x, name="var_1!")
+              } -> (%var_1!)
+            }
+
+
+        Output:
+            main(%x: (1, 3, 20, fp32)(Tensor)) {
+              block0() {
+                %var_1!: (1, 3, 20, fp32)(Tensor) = relu(x=%x, name="var_1_")
+              } -> (%var_1_)
+            }
+
+            main_2(%x: (1, 3, 20, fp32)(Tensor)) {
+              block0() {
+                %var_1!: (1, 3, 20, fp32)(Tensor) = relu(x=%x, name="var_1_")
+              } -> (%var_1_)
+            }
+
+        """
+
+        @mb.program(input_specs=[mb.TensorSpec(shape=(1, 3, 20))])
+        def prog(x):
+            z = mb.relu(x=x, name = "var_1!")
+            return z
+
+        @mb.program(input_specs=[mb.TensorSpec(shape=(1, 3, 20))])
+        def prog_2(x):
+            z = mb.relu(x=x, name = "var_1!")
+            return z
+
+        prog.add_function("main_2", prog_2.functions["main"])
+        PASS_REGISTRY["mil_backend::sanitize_name_strings"](prog)
+        block = prog.functions["main"]
+        assert block.find_ops(op_type="relu")[0].outputs[0].name == "var_1_"
+        assert prog["main"].outputs[0].name == "var_1_"
+        assert block.find_ops(op_type="relu")[0].name == "var_1_"
+        block = prog.functions["main_2"]
+        assert block.find_ops(op_type="relu")[0].outputs[0].name == "var_1_"
+        assert prog["main"].outputs[0].name == "var_1_"
+        assert block.find_ops(op_type="relu")[0].name == "var_1_"
+
 
 class TestPassFuseActivationSiLU:
     """
@@ -754,43 +891,3 @@ class TestPassFuseActivationSiLU:
             expected_output_shapes={block.outputs[0].name: tuple(x_shape)},
         )
 
-
-class TestHomogenizeInputDtypes:
-
-    @pytest.mark.parametrize(
-        ["op", "x_dtype", "y_dtype"],
-        [
-            ["add", "int32", "fp32"],
-            ["mul", "fp32", "int32"],
-            ["minimum", "int64", "int32"],
-            ["add", "int32", "fp16"],
-            ["add", "fp16", "int32"],
-            ["equal", "bool", "int32"],
-            ["mod", "int64", "fp16"],
-            ["not_equal", "fp32", "bool"],
-            ["pow", "fp16", "fp32"],
-            ["greater", "fp16", "fp32"],
-            ["matmul", "fp16", "int32"],
-        ]
-    )
-    def test_mixed_input_dtypes(self, op, x_dtype, y_dtype):
-        @mb.program(input_specs=[mb.TensorSpec(shape=(10, 10), dtype=string_to_builtin(x_dtype)),
-                                 mb.TensorSpec(shape=(10, 10), dtype=string_to_builtin(y_dtype))])
-        def prog(x, y):
-            x = getattr(mb, op)(x=x, y=y)
-            return x
-
-        assert get_op_types_in_program(prog) == [op]
-
-        _, _, block = apply_pass_and_basic_check(prog, "mil_backend::homogenize_input_dtypes")
-
-        assert get_op_types_in_program(prog) == ["cast", op]
-
-        promoted_dtype = promote_types(string_to_builtin(x_dtype), string_to_builtin(y_dtype))
-
-        # Asserting cast configuration
-        cast = block.find_ops(op_type="cast")[0]
-        assert cast.dtype.val == builtin_to_string(promoted_dtype)
-        assert len(cast.outputs) == 1
-        assert len(cast.outputs[0].child_ops) == 1
-        assert cast.outputs[0].child_ops[0].op_type == op
