@@ -4,7 +4,6 @@
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
 import os as _os
-import shutil as _shutil
 import stat as _stat
 import tempfile as _tempfile
 import warnings as _warnings
@@ -49,6 +48,17 @@ class MILFrontend:
     name = "milinternal"
 
     def __call__(self, model, *args, **kwargs):
+        specification_version = kwargs.get("specification_version", None)
+        if specification_version is not None:
+            max_opset_version, op = model._get_max_opset_version_and_op()
+            if max_opset_version > specification_version:
+                msg = (
+                    "Please update the minimum_deployment_target to {!s},"
+                    " since op {} is only available in opset {!s} or newer."
+                
+                ).format(max_opset_version, op.op_type, max_opset_version)
+                raise ValueError(msg)
+                
         if "inputs" in kwargs and kwargs["inputs"] is not None:
             inputs = kwargs["inputs"]
             if not isinstance(inputs, (list, tuple)):
@@ -219,6 +229,8 @@ def _mil_convert(
 
     if convert_to == 'milinternal':
         return mil_program # mil program
+    elif convert_to == 'milpython':
+        return proto # internal mil data structure
 
     elif convert_to == 'mlprogram':
         package_path = _create_mlpackage(proto, weights_dir, kwargs.get("package_dir"))
