@@ -34,10 +34,8 @@ from coremltools.converters.mil.mil import (
     types,
 )
 from coremltools.converters.mil.mil.input_type import (
-    BoolInputType,
     DefaultInputs,
     InputSpec,
-    IntInputType,
     TensorInputType,
 )
 from coremltools.converters.mil.mil.types.symbolic import is_symbolic
@@ -47,17 +45,21 @@ class TestCustomMatMul:
     # Define SSA Custom Op for Sparse MatMul
     # This will map to `custom_op` in SSA with binding information
     # to bind input spec to the custom implementation
-    @register_op(doc_str="Sparse MatMul Layer", is_custom_op=True)
+    @register_op(is_custom_op=True)
     class custom_sparse_matmul(Operation):
         # Defining input spec for current op
         input_spec = InputSpec(
-            x=TensorInputType(),
-            y=TensorInputType(),
-            transpose_x=BoolInputType(const=True, optional=True),
-            transpose_y=BoolInputType(const=True, optional=True),
-            x_is_sparse=BoolInputType(const=True, optional=True),
-            y_is_sparse=BoolInputType(const=True, optional=True),
+            x=TensorInputType(type_domain="T"),
+            y=TensorInputType(type_domain="T"),
+            transpose_x=TensorInputType(const=True, optional=True, type_domain=types.bool),
+            transpose_y=TensorInputType(const=True, optional=True, type_domain=types.bool),
+            x_is_sparse=TensorInputType(const=True, optional=True, type_domain=types.bool),
+            y_is_sparse=TensorInputType(const=True, optional=True, type_domain=types.bool),
         )
+        
+        type_domains = {
+            "T": (types.fp16, types.fp32),
+        }
 
         # Specifying binding for custom op for specifying inputs,
         # parameters required for creating custom op to be synced with Swift API
@@ -76,9 +78,6 @@ class TestCustomMatMul:
                 y_is_sparse=False,
                 )
 
-        def __init__(self, **kwargs):
-            super(TestCustomMatMul.custom_sparse_matmul, self).__init__(**kwargs)
-
         def type_inference(self):
             x_type = self.x.dtype
             x_shape = self.x.shape
@@ -86,11 +85,10 @@ class TestCustomMatMul:
             # For illustration purpose, assumming getting valid shape
             # Ideally, should consider transpose_?, ?_is_sparse parameters into consideration
             # for computing output shape
-            ret_shape = [x_shape[0], y_shape[1]]
             return types.tensor(x_type, [x_shape[0], y_shape[1]])
 
     # TensorFlow Sparse Matmul Op
-    @register_tf_op()
+    @register_tf_op
     def SparseMatMul(context, node):
         a = context[node.inputs[0]]
         b = context[node.inputs[1]]
@@ -121,7 +119,7 @@ class TestCustomMatMul:
         self, use_cpu_only, backend, transpose_a, transpose_b, a_is_sparse, b_is_sparse, b_is_const,
     ):
         if backend[0] == 'mlprogram':
-            pytest.xfail("Custom layer not supported with ML Program backend")
+            pytest.skip("Custom layer not supported with ML Program backend")
 
         rank = 2
         shape = list(np.random.randint(low=3, high=100, size=1)) * rank
@@ -178,15 +176,19 @@ class TestCustomTopK:
     @pytest.fixture(scope="class")
     def create_custom_TopK(self):
         # Defining SSA TopK Op
-        @register_op(doc_str="Custom TopK Layer", is_custom_op=True)
+        @register_op(is_custom_op=True)
         class custom_topk(Operation):
             input_spec = InputSpec(
-                x=TensorInputType(),
-                k=IntInputType(const=True, optional=True),
-                axis=IntInputType(const=True, optional=True),
-                sorted=BoolInputType(const=True, optional=True),
+                x=TensorInputType(type_domain="T"),
+                k=TensorInputType(const=True, optional=True, type_domain=types.int32),
+                axis=TensorInputType(const=True, optional=True, type_domain=types.int32),
+                sorted=TensorInputType(const=True, optional=True, type_domain=types.bool),
             )
-
+            
+            type_domains = {
+                "T": (types.fp16, types.fp32),
+            }
+            
             bindings = {
                 "class_name": "TopK",
                 "input_order": ["x"],
@@ -236,7 +238,6 @@ class TestCustomTopK:
 
         _TF_OPS_REGISTRY["TopKV2"] = default_tf_topk
 
-
     @pytest.mark.skipif(not testing_reqs._HAS_TF_1, reason=MSG_TF1_NOT_FOUND)
     @pytest.mark.parametrize(
         "use_cpu_only, backend, rank, k",
@@ -245,7 +246,7 @@ class TestCustomTopK:
     @pytest.mark.usefixtures("create_custom_TopK")
     def test_tf(self, use_cpu_only, backend, rank, k):
         if backend[0] == 'mlprogram':
-            pytest.xfail("Custom layer not supported with ML Program backend")
+            pytest.skip("Custom layer not supported with ML Program backend")
 
         shape = np.random.randint(low=3, high=6, size=rank)
         with tf.Graph().as_default() as graph:
@@ -267,5 +268,5 @@ class TestCustomTopK:
                 k == layers[-1].custom.parameters["k"].intValue
             ), "Incorrect parameter value k"
             assert (
-                True == layers[-1].custom.parameters["sorted"].boolValue
+                layers[-1].custom.parameters["sorted"].boolValue is True
             ), "Incorrect parameter value for Sorted"
