@@ -69,7 +69,9 @@ from coremltools.converters.mil.frontend.tensorflow.test.test_ops import (
     TestExpandDims,
     TestFill,
     TestGather,
+    TestGelu,
     TestIdentity,
+    TestIdentityN,
     TestImageResizing,
     TestIsFinite,
     TestL2Normalization,
@@ -100,6 +102,7 @@ from coremltools.converters.mil.frontend.tensorflow.test.test_ops import (
     TestSize,
     TestSliceByIndex,
     TestSliceBySize,
+    TestSoftmaxCrossEntropyWithLogits,
     TestSpaceToBatchND,
     TestSpaceToDepth,
     TestSplit,
@@ -138,7 +141,7 @@ class TestImageResample(TensorFlow2BaseTest):
         self, use_cpu_only, backend, data_warp_shapes,
     ):
         if backend[0] == "neuralnetwork":
-            pytest.xfail("nn backend not supported")
+            pytest.skip("nn backend not supported")
 
         tfa = pytest.importorskip("tensorflow_addons")
 
@@ -156,7 +159,7 @@ class TestImageResample(TensorFlow2BaseTest):
         ]
         input_dict = dict(zip(inputs, input_values))
         self.run_compare_tf2(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend,
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend,
         )
 
 
@@ -187,7 +190,7 @@ class TestImageTransform(TensorFlow2BaseTest):
     def test(self, use_cpu_only, backend, transforms, interpolation, shapes):
         x_shape, output_shape = shapes
         if backend[0] == "neuralnetwork":
-            pytest.xfail("nn backend not supported")
+            pytest.skip("nn backend not supported")
 
         tfa = pytest.importorskip("tensorflow_addons")
 
@@ -206,7 +209,7 @@ class TestImageTransform(TensorFlow2BaseTest):
         ]
         input_dict = dict(zip(inputs, input_values))
         self.run_compare_tf2(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend,
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend,
         )
 
 
@@ -225,7 +228,7 @@ class TestActivationSiLU(TensorFlow2BaseTest):
     )
     def test(self, use_cpu_only, backend, rank, tf_op):
         if backend[0] == "neuralnetwork":
-            pytest.xfail("nn backend not supported")
+            pytest.skip("nn backend not supported")
 
         x_shape = tuple(np.random.randint(low=1, high=4, size=rank))
 
@@ -239,7 +242,7 @@ class TestActivationSiLU(TensorFlow2BaseTest):
         ]
         input_dict = dict(zip(inputs, input_values))
         self.run_compare_tf2(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend,
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend,
         )
 
 
@@ -273,12 +276,9 @@ class TestResizeNearestNeighbor(TensorFlow2BaseTest):
                 return
             if target_shape[-2] % input_shape[-2] != 0:
                 return
-
-        if not use_cpu_only and not half_pixel_centers and backend[0] == "mlprogram":
-            # use_cpu_only == False & half_pixel_centers == False, & backend[0] == mlprogram
-            # then there are numerical errors
-            pytest.xfail("rdar://78321005")
-
+                
+        if backend[0] == "mlprogram" and not use_cpu_only and not half_pixel_centers:
+            pytest.xfail("rdar://97399545 (TestResizeNearestNeighbor failing on mlprogram + GPU + half_pixel_centers=False)")
 
         @make_tf_graph([input_shape])
         def build_model(x):
@@ -293,7 +293,7 @@ class TestResizeNearestNeighbor(TensorFlow2BaseTest):
         input_values = [random_gen(input_shape, -100, 100)]
         input_dict = dict(zip(inputs, input_values))
         self.run_compare_tf2(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend,
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend,
         )
 
     @pytest.mark.parametrize(
@@ -302,7 +302,7 @@ class TestResizeNearestNeighbor(TensorFlow2BaseTest):
     )
     def test_keras_layer(self, use_cpu_only, backend, size):
         if backend[0] == "neuralnetwork":
-            pytest.xfail("nn backend not supported")
+            pytest.skip("nn backend not supported")
 
         x_shape = tuple(np.random.randint(low=1, high=4, size=4))
 
@@ -316,7 +316,7 @@ class TestResizeNearestNeighbor(TensorFlow2BaseTest):
         input_values = [random_gen(x_shape, -100, 100)]
         input_dict = dict(zip(inputs, input_values))
         self.run_compare_tf2(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend,
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend,
         )
 
     @pytest.mark.parametrize(
@@ -329,14 +329,11 @@ class TestResizeNearestNeighbor(TensorFlow2BaseTest):
         ),
     )
     def test_tf_image_resize(self, use_cpu_only, backend, size, method):
-        if backend[0] == "mlprogram" and not use_cpu_only:
-            pytest.xfail("rdar://78343225 ((MIL GPU) Core ML Tools Unit Test failures [numerical error])")
-
         if backend[0] == "mlprogram" and size == (1, 1):
             pytest.xfail("rdar://79699954 (Nearest neighbor resize numerical mismatch when output size is (1,1))")
 
         if backend[0] == "neuralnetwork":
-            pytest.xfail("nn backend not supported")
+            pytest.skip("nn backend not supported")
 
         x_shape = tuple(np.random.randint(low=1, high=3, size=4))
 
@@ -350,7 +347,7 @@ class TestResizeNearestNeighbor(TensorFlow2BaseTest):
         ]
         input_dict = dict(zip(inputs, input_values))
         self.run_compare_tf2(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend,
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend,
         )
 
 
@@ -393,7 +390,7 @@ class TestNormalizationTF2(TensorFlowBaseTest):
             model,
             input_dict,
             outputs,
-            use_cpu_only=use_cpu_only,
+            use_cpu_for_conversion=use_cpu_only,
             backend=backend,
             atol=1e-2,
             rtol=1e-3,
@@ -433,7 +430,7 @@ class TestElementWiseBinaryTF2(TensorFlowBaseTest):
         input_dict = dict(zip(inputs, input_values))
 
         TensorFlowBaseTest.run_compare_tf(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend
         )
 
 
@@ -454,7 +451,7 @@ class TestControlFlowFromAutoGraph(TensorFlowBaseTest):
         input_values = [np.array([0.7], dtype=np.float32)]
         input_dict = dict(zip(inputs, input_values))
         TensorFlowBaseTest.run_compare_tf(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend
         )
 
     @pytest.mark.parametrize(
@@ -473,16 +470,13 @@ class TestControlFlowFromAutoGraph(TensorFlowBaseTest):
         input_values = [np.array([2], dtype=np.float32)]
         input_dict = dict(zip(inputs, input_values))
         TensorFlowBaseTest.run_compare_tf(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend
         )
 
     @pytest.mark.parametrize(
         "use_cpu_only, backend", itertools.product([True, False], backends)
     )
     def test_if_binary_add_if_else_mul(self, use_cpu_only, backend):
-        if backend[0] == "mlprogram":
-            pytest.xfail("rdar://81983176 (MLProgram Failure: Mismatched elements)")
-
         @make_tf_graph([(1,), (1,)])
         def build_model(x, y):
             if x > y:
@@ -498,7 +492,7 @@ class TestControlFlowFromAutoGraph(TensorFlowBaseTest):
         ]
         input_dict = dict(zip(inputs, input_values))
         TensorFlowBaseTest.run_compare_tf(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend
         )
 
     @pytest.mark.parametrize(
@@ -517,7 +511,7 @@ class TestControlFlowFromAutoGraph(TensorFlowBaseTest):
         input_values = [np.array([2.0], dtype=np.float32)]
         input_dict = dict(zip(inputs, input_values))
         TensorFlowBaseTest.run_compare_tf(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend
         )
 
     @pytest.mark.parametrize(
@@ -536,7 +530,7 @@ class TestControlFlowFromAutoGraph(TensorFlowBaseTest):
         input_values = [np.array([2.0], dtype=np.float32)]
         input_dict = dict(zip(inputs, input_values))
         TensorFlowBaseTest.run_compare_tf(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend
         )
 
     @pytest.mark.parametrize(
@@ -558,7 +552,7 @@ class TestControlFlowFromAutoGraph(TensorFlowBaseTest):
         input_values = [np.array([9.0], dtype=np.float32)]
         input_dict = dict(zip(inputs, input_values))
         TensorFlowBaseTest.run_compare_tf(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend
         )
 
 @pytest.mark.xfail(reason="rdar://76293949 (TF2 unit test InvalidArgumentError)", run=False)
@@ -593,7 +587,7 @@ class TestTensorList(TensorFlowBaseTest):
         ]
         input_dict = dict(zip(inputs, input_values))
         TensorFlowBaseTest.run_compare_tf(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend
         )
 
     @pytest.mark.parametrize(
@@ -622,7 +616,7 @@ class TestTensorList(TensorFlowBaseTest):
         input_values = [np.array([[3.14], [6.17], [12.14]], dtype=np.float32)]
         input_dict = dict(zip(inputs, input_values))
         TensorFlowBaseTest.run_compare_tf(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend
         )
 
     @pytest.mark.parametrize(
@@ -655,7 +649,7 @@ class TestTensorList(TensorFlowBaseTest):
         ]
         input_dict = dict(zip(inputs, input_values))
         TensorFlowBaseTest.run_compare_tf(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend
         )
 
     @pytest.mark.parametrize(
@@ -684,7 +678,7 @@ class TestTensorList(TensorFlowBaseTest):
         input_values = [np.array([[3.14], [6.17], [12.14]], dtype=np.float32)]
         input_dict = dict(zip(inputs, input_values))
         TensorFlowBaseTest.run_compare_tf(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend
         )
 
     @pytest.mark.parametrize(
@@ -709,5 +703,5 @@ class TestTensorList(TensorFlowBaseTest):
         input_values = [np.random.rand(3, 1, 8).astype(np.float32)]
         input_dict = dict(zip(inputs, input_values))
         TensorFlowBaseTest.run_compare_tf(
-            model, input_dict, outputs, use_cpu_only=use_cpu_only, backend=backend
+            model, input_dict, outputs, use_cpu_for_conversion=use_cpu_only, backend=backend
         )
