@@ -5,10 +5,11 @@
 
 from distutils.version import StrictVersion as _StrictVersion
 import itertools
-import numpy as np
-import pytest
 import random
 
+import numpy as np
+import platform
+import pytest
 
 import coremltools as ct
 from coremltools._deps import _get_version
@@ -19,7 +20,10 @@ from coremltools.converters.mil.frontend.tensorflow2.test.testing_utils import (
 from coremltools.converters.mil.frontend.tensorflow.test.testing_utils import (
     TensorFlowBaseTest
 )
-from coremltools.converters.mil.testing_utils import random_gen
+from coremltools.converters.mil.testing_utils import (
+    get_op_types_in_program,
+    random_gen,
+)
 from ..._utils import is_symbolic_dim_in_prog
 
 TensorFlowBaseTest.run_compare_tf_keras = \
@@ -28,6 +32,9 @@ backends = testing_reqs.backends
 
 tf = pytest.importorskip("tensorflow", minversion="2.1.0")
 import tensorflow as _tf  # should be after pytest.importorskip checks
+from tensorflow.keras import Input
+from tensorflow.keras.models import Model
+from tensorflow.keras.layers import Conv2D, GlobalMaxPooling2D
 
 
 class TestActivation(TensorFlowBaseTest):
@@ -98,6 +105,29 @@ class TestActivation(TensorFlowBaseTest):
             **kwargs
         )
 
+    @pytest.mark.parametrize("backend", backends)
+    def test_conv2d_prelu_fusion(self, backend):
+        x_shape = (1, 10, 10, 32)
+        x = tf.keras.Input(batch_input_shape=x_shape)  # (B, H, W, C)
+        x1 = tf.keras.layers.Conv2D(16, kernel_size=1)(x)
+        x1 =  tf.keras.layers.PReLU(alpha_initializer='glorot_uniform', shared_axes=[1, 2])(x1)
+        x1 = tf.keras.layers.Conv2D(16, kernel_size=1)(x1)
+        x1 = tf.keras.layers.PReLU(alpha_initializer='glorot_uniform', shared_axes=[1, 2])(x1)
+        keras_model = tf.keras.Model(inputs=x, outputs=x1)
+
+        res = TensorFlowBaseTest.run_compare_tf_keras(
+            keras_model,
+            [random_gen(x_shape, -1, 1)],
+            use_cpu_only=True,
+            backend=backend,
+        )
+        coreml_model = res[1]
+        mil_prog = coreml_model._get_mil_internal()
+        # assert that "prelu" ops are present in the mil program,
+        # which should be if "fuse_prelu" pass worked correctly
+        assert len(mil_prog.find_ops(op_type="prelu")) == 2
+        assert "relu" not in get_op_types_in_program(mil_prog)
+
 
 class TestBinary(TensorFlowBaseTest):
     @pytest.mark.parametrize(
@@ -419,7 +449,7 @@ class TestConvolution(TensorFlowBaseTest):
         batch_size,
     ):
         s1, s2, k1, k2 = spatial_dim_and_ks
-        c_in, c_out = 2, 6
+        c_in = 2
 
         if len(strides) != np.sum(strides) and len(dilations) != np.sum(dilations):
             # TF produces incorrect output for non-one strides + dilations
@@ -466,10 +496,6 @@ class TestConvolution(TensorFlowBaseTest):
         backend,
         padding,
     ):
-        from tensorflow.keras import Input
-        from tensorflow.keras.models import Model
-        from tensorflow.keras.layers import Conv2D, GlobalMaxPooling2D
-
         # Test same padding
         input_layer = Input(batch_size=1, shape=(None, None, 1))
         layer = Conv2D(
@@ -482,7 +508,7 @@ class TestConvolution(TensorFlowBaseTest):
         model = Model(inputs=[input_layer], outputs=[output_layer])
         TensorFlowBaseTest.run_compare_tf_keras(
             model,
-            [random_gen((1, 80, 40 ,1), rand_min=-10, rand_max=10)],
+            [random_gen((1, 80, 40, 1), rand_min=-10, rand_max=10)],
             use_cpu_only=use_cpu_only,
             backend=backend,
         )
@@ -602,6 +628,16 @@ class TestConvTranspose(TensorFlowBaseTest):
         dilations,
         batch_size,
     ):
+        if (platform.machine() == "arm64" and
+            backend == ("mlprogram", "fp16") and 
+            op == tf.keras.layers.Conv3DTranspose and
+            padding == "valid" and
+            spatial_dim_and_ks == (7, 11, 12, 1, 2, 2) and
+            strides == (2, 3, 3) and
+            batch_size == 3
+        ):
+           pytest.xfail("rdar://98015195 ([M1 native tests] Some MIL unittests are failing M1 native)")
+
         s1, s2, s3, k1, k2, k3 = spatial_dim_and_ks
         c_in, c_out = 2, 3
         input_shape = None
@@ -855,6 +891,9 @@ class TestBatchNormalization(TensorFlowBaseTest):
                 )
             ]
         )
+        random_weights = np.random.rand(4, shape[axis])
+        model.layers[0].set_weights(random_weights)
+
         TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-10, rand_max=10)],
@@ -883,6 +922,9 @@ class TestBatchNormalization(TensorFlowBaseTest):
                 )
             ]
         )
+        random_weights = np.random.rand(4, shape[axis])
+        model.layers[0].set_weights(random_weights)
+
         TensorFlowBaseTest.run_compare_tf_keras(
             model,
             [random_gen(shape, rand_min=-10, rand_max=10)],
@@ -1247,9 +1289,9 @@ class TestRecurrent(TensorFlowBaseTest):
         h0 = tf.keras.layers.Input(shape=(512,))
         c0 = tf.keras.layers.Input(shape=(512,))
         out, hn, cn = tf.keras.layers.LSTM(512,
-                                        return_sequences=True,
-                                        return_state=True,
-                                        recurrent_activation='sigmoid')(inp)
+                                           return_sequences=True,
+                                           return_state=True,
+                                           recurrent_activation='sigmoid')(inp)
         model = tf.keras.models.Model(inputs=[inp, h0, c0], outputs=[out, hn, cn])
         batch_size = 2
         TensorFlowBaseTest.run_compare_tf_keras(
@@ -1281,19 +1323,16 @@ class TestRecurrent(TensorFlowBaseTest):
         def _test_for_symbolic_shapes(keras_input_shape, input_shape_for_conversion, are_symbols_expected):
             keras_model = _get_keras_simple_lstm_model(keras_input_shape)
             res = TensorFlowBaseTest.run_compare_tf_keras(
-                        keras_model,
-                        [
-                            random_gen((1, 32, 10), -1, 1),
-                        ],
-                        inputs_for_conversion=[ct.TensorType(shape=input_shape_for_conversion)],
-                        use_cpu_only=use_cpu_only,
-                        backend=backend,
-                    )
+                keras_model,
+                [random_gen((1, 32, 10), -1, 1)],
+                inputs_for_conversion=[ct.TensorType(shape=input_shape_for_conversion)],
+                use_cpu_only=use_cpu_only,
+                backend=backend,
+            )
             coreml_model = res[1]
             mil_prog = coreml_model._get_mil_internal()
             assert is_symbolic_dim_in_prog(mil_prog) == are_symbols_expected
 
-
         _test_for_symbolic_shapes(keras_input_shape=(1, 32, 10),
                                   input_shape_for_conversion=(1, 32, 10),
                                   are_symbols_expected=False)
@@ -1316,6 +1355,79 @@ class TestRecurrent(TensorFlowBaseTest):
                                       input_shape_for_conversion=(ct.RangeDim(1, 10), ct.RangeDim(16, 64), 10),
                                       are_symbols_expected=True)
 
+    @pytest.mark.parametrize(
+        "use_cpu_only, tf_raw_lstm_op, backend",
+        itertools.product([True, False], [tf.raw_ops.BlockLSTM, tf.raw_ops.BlockLSTMV2], backends,),
+    )
+    def test_lstm_block_fused_op(self, use_cpu_only, tf_raw_lstm_op, backend):
+        '''
+        Define a model with custom LSTM ops that uses tf.raw_ops.BlockLSTM / tf.raw_ops.BlockLSTMV2
+        and verify that it converts to a fused lstm op.
+
+        %x (shape: (Seq, Batch, idim) == (5, 2, 4))
+        %x1 = LSTM(h=10) (%input) # shape = (5, 2, 10)
+        %x2 = LSTM(h=20) (%x1) # shape = (5, 2, 20)
+        %x3 = slice()(%x2) # shape = (1, 2, 20), to get the final seq value
+        %x4 = reshape((1, -1)) (%x3) # shape = (1, 40)
+        %x5 = Dense(h=3)(%x4) # shape = (1, 3)
+        '''
+        class CustomLSTM(tf.keras.layers.Layer):
+            def __init__(self, num_units, max_seq_length, batch_size):
+                super(CustomLSTM, self).__init__()
+                self.hidden_dim = num_units
+                self.seq_length = max_seq_length
+                self.batch_size = batch_size
+
+            def build(self, input_shape):
+                input_dim = input_shape[-1]
+                self.w = self.add_weight(
+                    shape=(input_dim + self.hidden_dim, 4 * self.hidden_dim),
+                    initializer="random_normal",
+                    trainable=True,
+                )
+                self.b = self.add_weight(shape=(4 * self.hidden_dim,), initializer="random_normal", trainable=True)
+                self.init_h = tf.constant(np.zeros((self.batch_size, self.hidden_dim)).astype(np.float32))
+                self.init_c = tf.constant(np.zeros((self.batch_size, self.hidden_dim)).astype(np.float32))
+
+            def call(self, inputs):
+                _, output_state, _, _, _, _, output = tf_raw_lstm_op(
+                    seq_len_max=self.seq_length,
+                    x=inputs,
+                    cs_prev=self.init_c,
+                    h_prev=self.init_h,
+                    w=self.w,
+                    wci=tf.constant(np.zeros((self.hidden_dim)).astype(np.float32)),
+                    wcf=tf.constant(np.zeros((self.hidden_dim)).astype(np.float32)),
+                    wco=tf.constant(np.zeros((self.hidden_dim)).astype(np.float32)),
+                    b=self.b,
+                )
+                return output
+
+        input_dim = 4
+        seq_length = 5
+        batch_size = 2
+        x_shape = (seq_length, batch_size, input_dim)
+        hidden_dim_1 = 10
+        hidden_dim_2 = 20
+
+        x = tf.keras.Input(batch_input_shape=x_shape)  # (5, 2, 4)
+        x1 = CustomLSTM(num_units=hidden_dim_1, max_seq_length=seq_length, batch_size=batch_size)(x)  # (5, 2, 10)
+        x2 = CustomLSTM(num_units=hidden_dim_2, max_seq_length=seq_length, batch_size=batch_size)(x1)  # (5, 2, 20)
+        x3 = tf.slice(x2, begin=[4, 0, 0], size=[1, 2, 20])  # (1, 2, 20)
+        x4 = tf.reshape(x3, shape=(1, -1))  # (1, 40)
+        x5 = tf.keras.layers.Dense(3)(x4)  # (1, 3)
+        keras_model = tf.keras.Model(inputs=x, outputs=x5)
+
+        res = TensorFlowBaseTest.run_compare_tf_keras(
+            keras_model,
+            [random_gen(x_shape, -1, 1)],
+            use_cpu_only=use_cpu_only,
+            backend=backend,
+        )
+        coreml_model = res[1]
+        mil_prog = coreml_model._get_mil_internal()
+        # assert that "lstm" ops are present in the mil program
+        assert len(mil_prog.find_ops(op_type="lstm")) == 2
 
 
 class TestRepeatVector(TensorFlowBaseTest):
@@ -1504,7 +1616,10 @@ class TestUpSampling(TensorFlowBaseTest):
                     assert len(layer.upsample.fractionalScalingFactor) == 0
 
 class TestGelu(TensorFlowBaseTest):
-    @pytest.mark.xfail(_get_version(_tf.__version__) < _StrictVersion("2.4.0"), reason="Gelu is a new layer for tf 2.4.0 and above.")
+    @pytest.mark.skipif(
+        _get_version(_tf.__version__) < _StrictVersion("2.4.0"),
+        reason="Gelu is a new layer for tf 2.4.0 and above."
+    )
     @pytest.mark.parametrize(
         "use_cpu_only, backend, rank, approximate",
         itertools.product(
