@@ -5,9 +5,11 @@
 
 from collections import OrderedDict
 import logging as _logging
+import numpy as _np
 import torch as _torch
 
 from coremltools._deps import version_lt
+from coremltools.converters.mil._deployment_compatibility import AvailableTarget as _target
 from coremltools.converters.mil.input_types import ImageType
 from coremltools.converters.mil.mil import (
     Builder as mb,
@@ -27,24 +29,27 @@ from .torchir_passes import (
     remove_getattr_nodes
 )
 from .ssa_passes.torch_passes import torch_passes
+from .._utils import get_output_names
 
 torch_to_mil_types = {
     _torch.bool: types.bool,
     _torch.float16: types.fp16,
     _torch.float32: types.fp32,
-    _torch.float64: types.fp64,
+    _torch.float64: types.fp32,
     _torch.int32: types.int32,
-    _torch.int64: types.int64,
+    _torch.int64: types.int32,
 }
 
 mil_to_torch_types = {v: k for k, v in torch_to_mil_types.items()}
 
 
 class TranscriptionContext:
-    """Maintains a map from torch operations to their MIL values
-        while building the graph. Can be used to process subgraphs recursively
-        by pushing new context when stepping into a subgraph and popping that
-        context when stepping out."""
+    """
+    Maintains a map from torch operations to their MIL values
+    while building the graph. Can be used to process subgraphs recursively
+    by pushing new context when stepping into a subgraph and popping that
+    context when stepping out.
+    """
 
     def __init__(self, name=None):
         self.name = name if name else ""
@@ -65,9 +70,10 @@ class TranscriptionContext:
         self._current_graph[-1][torch_name] = ssa_var
 
     def __getitem__(self, torch_name):
-        """ Lookup a name in the context. Note that since nested blocks must be
-            able to access anything that was defined before them, we have to
-            search all contexts for a name, starting with the most local scope.
+        """
+        Lookup a name in the context. Note that since nested blocks must be
+        able to access anything that was defined before them, we have to
+        search all contexts for a name, starting with the most local scope.
         """
         for idx in reversed(range(len(self._current_graph))):
             current_graph = self._current_graph[idx]
@@ -120,32 +126,38 @@ class TranscriptionContext:
 
 
 class TorchConverter:
-    """Class that handles conversion of pytorch models represented in TorchScript
+    """
+    Class that handles conversion of pytorch models represented in TorchScript
     format to the MIL format.
 
     Models passed to the @TorchConverter go from:
     TorchScript -> Expanded/Optimized Torch IR -> Internal Graph -> CoreML SSA
     The internal graph representation was added to make testing easier.
-
-    Arguments:
-        torchscript: torch.jit.ScriptModule object representing the model to convert.
-        inputs: Input values and optional names. See kwarg in load.py for full description.
-        outputs: Names of the graph's outputs. See kwarg in load.py for full description.
-        cut_at_symbols: A list of internal symbol name strings. Graph conversion will
-            terminate once these symbols have been generated. For debugging use
-            only. See kwarg in load.py.
     """
 
     def __init__(
-        self, torchscript, inputs, outputs=None, cut_at_symbols=None,
+        self, torchscript, inputs, outputs=None, cut_at_symbols=None, opset_version=None
     ):
+        """
+        Arguments:
+            torchscript: torch.jit.ScriptModule object representing the model to convert.
+            inputs: Input values and optional names. See kwarg in load.py for full description.
+            outputs: List of outputs as ct.InputType. See kwarg in load.py for full description.
+            cut_at_symbols: A list of internal symbol name strings. Graph conversion will
+                terminate once these symbols have been generated. For debugging use
+                only. See kwarg in load.py.
+            opset_version: An int represents the coreml opset version
+        """
+
         assert isinstance(torchscript, _torch.jit.ScriptModule)
         self.inputs = inputs
         for idx, inp in enumerate(self.inputs):
             if isinstance(inp, ImageType) and self.inputs[idx].channel_first is None:
                 self.inputs[idx].channel_first = True
         self.torchscript = torchscript
-        self.output_names = outputs
+        self.outputs = outputs
+        self.output_names = get_output_names(self.outputs)
+        self.opset_version = _target(opset_version) if opset_version is not None else None
         self.context = TranscriptionContext()
         raw_graph, params_dict = self._expand_and_optimize_ir(self.torchscript)
         self.params_dict = params_dict
@@ -163,12 +175,15 @@ class TorchConverter:
             p(self.graph)
         self.inputs = [v for v in self.graph.inputs.values()]
         self.torch_passes = torch_passes
+        self._prog = Program()
 
     @staticmethod
     def _check_ops(graph):
-        """ Returns the set of ops in @graph that are implemented, and the set
-            for which no conversion function is registered. @graph can be
-            either InternalTorchIRGraph or InternalTorchIRBlock."""
+        """
+        Returns the set of ops in @graph that are implemented, and the set
+        for which no conversion function is registered. @graph can be
+        either InternalTorchIRGraph or InternalTorchIRBlock.
+        """
         implemented_ops = set()
         missing_ops = set()
         for node in graph.nodes:
@@ -185,7 +200,8 @@ class TorchConverter:
 
     @staticmethod
     def _create_placeholder(_input):
-        """Converts an InputType into a Placeholder.
+        """
+        Converts an InputType into a Placeholder.
 
         _input: TensorType
         """
@@ -194,21 +210,24 @@ class TorchConverter:
         return mb.placeholder(shape, dtype=dtype)
 
     def check_ops(self):
-        """ Returns the set of ops in @self.graph that are implemented, and
-            the set for which no conversion function is registered."""
+        """
+        Returns the set of ops in @self.graph that are implemented, and
+        the set for which no conversion function is registered.
+        """
         return TorchConverter._check_ops(self.graph)
 
     def convert_const(self):
         for name, val in self.graph.params.items():
+            if val.dtype == _np.uint8:
+                val = val.astype(_np.int32)
             const = mb.const(val=val, name=name)
             self.context.add(const)
 
     def convert(self):
-
         _logging.info("Converting graph.")
 
         # This will hold the converted model.
-        prog = Program()
+        prog = self._prog
 
         # Construct placeholder for input to ssa function
         # This is where input renaming occurs
@@ -223,7 +242,7 @@ class TorchConverter:
         prog.set_main_input_types(tuple(self.inputs))
 
         # Initialize the SSA for conversion
-        with Function(ssa_func_inputs) as ssa_func:
+        with Function(ssa_func_inputs, opset_version=self.opset_version) as ssa_func:
 
             # Map internal @self.graph.inputs to user specified @ssa_func_inputs
             # If @self.graph.inputs == @ssa_func_inputs this just adds the inputs
@@ -231,7 +250,18 @@ class TorchConverter:
             for internal_name, users_name in zip(
                 self.graph.inputs.keys(), ssa_func_inputs.keys()
             ):
-                self.context.add(ssa_func.inputs[users_name], torch_name=internal_name)
+                input_var = ssa_func.inputs[users_name]
+                if (types.is_tensor(input_var.sym_type) or types.is_scalar(input_var.sym_type)) \
+                    and (input_var.dtype == types.fp16 or input_var.dtype == types.fp64):
+                    # cast the input var to float32
+                    # We need to do this because the type inference is very buggy when started from
+                    # float16/float64 typed inputs. Until that is fixed in the following radar
+                    # we cast all inputs of type float16/float64 to float32 as the first step.
+                    # These casts will later get removed, if compute_precision=Float16 is
+                    # provided, which will cause the FP16ComputePrecision pass to run.
+                    # TODO: remove this when this radar is fixed: rdar://93731970
+                    input_var = mb.cast(x=input_var, dtype="fp32")
+                self.context.add(input_var, torch_name=internal_name)
 
             self.convert_const()
 
@@ -249,13 +279,23 @@ class TorchConverter:
             graph_outputs = [g for g in graph_outputs if g is not None]
 
             # Output renaming occurs
+            if self.outputs is not None:
+                if len(self.outputs) != len(graph_outputs):
+                    msg = "Number of outputs provided, {}, do not match the number of outputs detected in the model, {}."
+                    raise ValueError(msg.format(
+                        len(self.outputs),
+                        len(graph_outputs),
+                    ))
             if self.output_names:
                 for index, var in enumerate(graph_outputs):
-                    output_rename = self.output_names[index]
-                    var.name = output_rename
+                    if self.output_names[index] is not None:
+                        output_rename = self.output_names[index]
+                        var.name = output_rename
 
             ssa_func.set_outputs(graph_outputs)
             prog.add_function("main", ssa_func)
+            if self.outputs is not None:
+                prog.set_main_output_types(self.outputs)
         self.torch_passes(prog)
         return prog
 
@@ -390,10 +430,10 @@ class TorchConverter:
 
         return graph, params_dict
 
-
     @staticmethod
     def _expand_and_optimize_ir(torchscript):
-        """Given a torch.jit.ScriptModule, convert it to a optimized
+        """
+        Given a torch.jit.ScriptModule, convert it to a optimized
         torch._C.Graph and dict of model parameter's names to tensors.
         """
         graph = torchscript.forward.graph
@@ -410,15 +450,6 @@ class TorchConverter:
         _torch._C._jit_pass_dce(graph)
         # From PyTorch code: checks well-formedness and invariants of graph.
         _torch._C._jit_pass_lint(graph)
-        # From PyTorch code: remove all in-place ops and replace them with
-        # out-of-place equivalents.
-        # e.g.
-        #   %foo = aten::add_(%foo, %n)
-        # becomes
-        #   %foo.2 = aten::add(%foo, %n)
-        _torch._C._jit_pass_remove_inplace_ops(graph)
-        _torch._C._jit_pass_dce(graph)
-        _torch._C._jit_pass_lint(graph)
         # Replaces a couple specific ops patterns (add, sub, mul, div, chunk).
         if version_lt(_torch, '1.6.0'):
             _torch._C._jit_pass_canonicalize_ops(graph)
