@@ -5,15 +5,11 @@
 
 from coremltools.converters.mil.mil import get_new_symbol, Operation, types
 from coremltools.converters.mil.mil.input_type import (
-    BoolInputType,
-    BoolTensorInputType,
     DefaultInputs,
     InputSpec,
-    IntOrFloatInputType,
-    IntTensorInputType,
     TensorInputType
 )
-from coremltools.converters.mil.mil.ops.defs.tensor_transformation import _solve_slice_by_index_shape
+from coremltools.converters.mil.mil.ops.defs._utils import solve_slice_by_index_shape
 from coremltools.converters.mil.mil.ops.registry import SSAOpRegistry
 from coremltools.converters.mil.mil.types.symbolic import is_compatible_symbolic_vector
 
@@ -31,7 +27,7 @@ register_op = SSAOpRegistry.register_op
 
 # torch_upsample_nearest_neighbor is dealing with upsample layer which has flexible input shape,
 # and recompute_scale_factor is set to True in the original torch layer.
-@register_op(doc_str="", namespace="torch")
+@register_op(namespace="torch")
 class torch_upsample_nearest_neighbor(Operation):
     """
     Upsample the spatial dimensions (last two dimensions) of the input by
@@ -42,8 +38,8 @@ class torch_upsample_nearest_neighbor(Operation):
 
     Parameters
     ----------
-    x: tensor<[\*D, H1, W1],T>  (Required)
-        * Must be at least rank ``3``.
+    x: tensor<[b, C, H1, W1],T>  (Required)
+        * Must be rank ``4``.
     output_height: i32
         * Output height for the height dimension.
     output_width: i32
@@ -51,37 +47,38 @@ class torch_upsample_nearest_neighbor(Operation):
 
     Returns
     -------
-    tensor<[\*D, H2, W2],T>
+    tensor<[b, C, H2, W2],T>
         * Tensor with same type as the input.
         * ``H2`` = output_height
         * ``W2`` = output_width
 
     Attributes
     ----------
-    T: fp32
+    T: fp16, fp32
     """
     input_spec = InputSpec(
-        x=TensorInputType(),
-        output_height=IntOrFloatInputType(),
-        output_width=IntOrFloatInputType(),
+        x=TensorInputType(type_domain="T"),
+        output_height=TensorInputType(type_domain=types.int32),
+        output_width=TensorInputType(type_domain=types.int32),
     )
-
-    def __init__(self, **kwargs):
-        super(torch_upsample_nearest_neighbor, self).__init__(**kwargs)
+ 
+    type_domains = {
+        "T": (types.fp16, types.fp32),
+    }
 
     def type_inference(self):
-        if self.x.rank < 3:
+        if self.x.rank != 4:
             raise ValueError(
-                'input to the "torch_upsample_nearest_neighbor" op must have rank at least 3'
+                'input to the "torch_upsample_nearest_neighbor" op must have rank 4'
             )
         ret_shape = list(self.x.shape)
-        ret_shape[-1] = get_new_symbol()
-        ret_shape[-2] = get_new_symbol()
+        ret_shape[2] = get_new_symbol()
+        ret_shape[3] = get_new_symbol()
         return types.tensor(self.x.dtype, ret_shape)
 
 # torch_upsample_bilinear is dealing with upsample layer which has flexible input shape,
 # and recompute_scale_factor is set to True in the original torch layer.
-@register_op(doc_str="", namespace="torch")
+@register_op(namespace="torch")
 class torch_upsample_bilinear(Operation):
     """
     Upsample the spatial dimensions (last two dimensions) of the input by
@@ -92,8 +89,8 @@ class torch_upsample_bilinear(Operation):
 
     Parameters
     ----------
-    x: tensor<[\*D, H1, W1],T>  (Required)
-        * Must be rank ``3``.
+    x: tensor<[b, C, H1, W1], T>  (Required)
+        * Must be rank ``4``.
     output_height: i32
         * Output height for the height dimension.
     output_width: i32
@@ -103,42 +100,43 @@ class torch_upsample_bilinear(Operation):
 
     Returns
     -------
-    tensor<[\*D, H2, W2],T>
+    tensor<[b, C, H2, W2], T>
         * Tensor with same type as the input.
         * ``H2`` = output_height
         * ``W2`` = output_width
 
     Attributes
     ----------
-    T: fp32
+    T: fp16, fp32
     """
     input_spec = InputSpec(
-        x=TensorInputType(),
-        output_height=IntOrFloatInputType(),
-        output_width=IntOrFloatInputType(),
-        align_corners=BoolInputType(const=True, optional=True),
+        x=TensorInputType(type_domain="T"),
+        output_height=TensorInputType(type_domain=types.int32),
+        output_width=TensorInputType(type_domain=types.int32),
+        align_corners=TensorInputType(const=True, optional=True, type_domain=types.bool),
     )
 
+    type_domains = {
+        "T": (types.fp16, types.fp32),
+    }
+
     def default_inputs(self):
         return DefaultInputs(
             align_corners=True,
-            )
-
-    def __init__(self, **kwargs):
-        super(torch_upsample_bilinear, self).__init__(**kwargs)
+        )
 
     def type_inference(self):
-        if self.x.rank < 3:
+        if self.x.rank != 4:
             raise ValueError(
-                'input to the "torch_upsample_bilinear" op must have rank at least 3'
+                'input to the "torch_upsample_bilinear" op must have rank 4'
             )
         ret_shape = list(self.x.shape)
-        ret_shape[-1] = get_new_symbol()
-        ret_shape[-2] = get_new_symbol()
+        ret_shape[2] = get_new_symbol()
+        ret_shape[3] = get_new_symbol()
         return types.tensor(self.x.dtype, ret_shape)
 
 # torch_tensor_assign is dealing with the tensor assignment operation
-@register_op(doc_str="", namespace="torch")
+@register_op(namespace="torch")
 class torch_tensor_assign(Operation):
     """
     Method for tensor value assignment via indexing and slicing.
@@ -176,19 +174,23 @@ class torch_tensor_assign(Operation):
 
     Attributes
     ----------
-    T: fp32
+    T: fp16, fp32, i32
     """
 
     input_spec = InputSpec(
-        data=TensorInputType(),
-        updates=IntOrFloatInputType(),
-        begin=IntTensorInputType(const=True),
-        end=IntTensorInputType(const=True),
-        stride=IntTensorInputType(const=True, optional=True),
-        begin_mask=BoolTensorInputType(const=True, optional=True),
-        end_mask=BoolTensorInputType(const=True, optional=True),
-        squeeze_mask=BoolTensorInputType(const=True, optional=True),
+        data=TensorInputType(type_domain="T"),
+        updates=TensorInputType(type_domain="T"),
+        begin=TensorInputType(const=True, type_domain=types.int32),
+        end=TensorInputType(const=True, type_domain=types.int32),
+        stride=TensorInputType(const=True, optional=True, type_domain=types.int32),
+        begin_mask=TensorInputType(const=True, optional=True, type_domain=types.bool),
+        end_mask=TensorInputType(const=True, optional=True, type_domain=types.bool),
+        squeeze_mask=TensorInputType(const=True, optional=True, type_domain=types.bool),
     )
+    
+    type_domains = {
+        "T": (types.fp16, types.fp32, types.int32),
+    }
 
     def default_inputs(self):
         return DefaultInputs(
@@ -196,10 +198,7 @@ class torch_tensor_assign(Operation):
             begin_mask=None,
             end_mask=None,
             squeeze_mask=None,
-            )
-
-    def __init__(self, **kwargs):
-        super(torch_tensor_assign, self).__init__(**kwargs)
+        )
 
     def type_inference(self):
         # Verify the updates and the data slicing have the same shape
@@ -215,7 +214,7 @@ class torch_tensor_assign(Operation):
             self.squeeze_mask.val if self.squeeze_mask is not None else [False] * data_rank
         )
         data_shape = self.data.shape
-        expected_updates_shape = tuple(_solve_slice_by_index_shape(data_shape, begin, end, stride, begin_mask, end_mask, squeeze_mask))
+        expected_updates_shape = tuple(solve_slice_by_index_shape(data_shape, begin, end, stride, begin_mask, end_mask, squeeze_mask))
         if not is_compatible_symbolic_vector(expected_updates_shape, self.updates.shape):
             raise ValueError("The updates tensor should have shape {}. Got {}".format(expected_updates_shape, self.updates.shape))
         return self.data.sym_type
