@@ -19,16 +19,16 @@ from coremltools.converters.mil.frontend.torch.torch_op_registry import (
     _TORCH_OPS_REGISTRY as _TORCH_OPS_REG,
 )
 from coremltools.converters.mil.frontend.torch.ops import _get_inputs
+from coremltools.converters.mil.frontend.torch.ops import cosine_similarity as cosine_similarity_main
 from coremltools.converters.mil.mil import (
     Builder as mb,
     Operation,
     types
 )
 from coremltools.converters.mil.mil.input_type import (
-    BoolInputType,
     DefaultInputs,
     InputSpec,
-    TensorInputType
+    TensorInputType,
 )
 
 
@@ -38,23 +38,7 @@ default_cosine_similarity = _TORCH_OPS_REG.get("cosine_similarity", None)
 
 @register_torch_op(override=True)
 def cosine_similarity(context, node):
-    inputs = _get_inputs(context, node, expected=4)
-    dim = inputs[-2].val
-    eps = inputs[-1].val
-    xy = mb.mul(x=inputs[0], y=inputs[1])
-    sum_xy = mb.reduce_sum(x=xy, axes=[dim])
-
-    xx = mb.mul(x=inputs[0], y=inputs[0])
-    sum_xx = mb.reduce_sum(x=xx, axes=[dim])
-    yy = mb.mul(x=inputs[1], y=inputs[1])
-    sum_yy = mb.reduce_sum(x=yy, axes=[dim])
-
-    mul_sum_xy = mb.mul(x=sum_xx, y=sum_yy)
-    div_12 = mb.maximum(x=mul_sum_xy, y=eps * eps)
-    div_sqrt = mb.sqrt(x=div_12)
-
-    cs = mb.real_div(x=sum_xy, y=div_sqrt, name=node.name)
-    context.add(cs)
+    cosine_similarity_main(context, node)
 
 
 # Log custom Cosine Similarity conversion function
@@ -79,17 +63,21 @@ class TestCustomOp:
     # Define SSA Custom Op for Sparse MatMul
     # This will map to `custom_op` in SSA with binding information
     # to bind input spec to the custom implementation
-    @register_op(doc_str="Sparse MatMul Layer", is_custom_op=True)
+    @register_op(is_custom_op=True)
     class custom_torch_sparse_matmul(Operation):
         # Defining input spec for current op
         input_spec = InputSpec(
-            x=TensorInputType(),
-            y=TensorInputType(),
-            transpose_x=BoolInputType(const=True, optional=True),
-            transpose_y=BoolInputType(const=True, optional=True),
-            x_is_sparse=BoolInputType(const=True, optional=True),
-            y_is_sparse=BoolInputType(const=True, optional=True),
+            x=TensorInputType(type_domain="T"),
+            y=TensorInputType(type_domain="T"),
+            transpose_x=TensorInputType(const=True, optional=True, type_domain=types.bool),
+            transpose_y=TensorInputType(const=True, optional=True, type_domain=types.bool),
+            x_is_sparse=TensorInputType(const=True, optional=True, type_domain=types.bool),
+            y_is_sparse=TensorInputType(const=True, optional=True, type_domain=types.bool),
         )
+        
+        type_domains = {
+            "T": (types.fp16, types.fp32),
+        }
 
         def default_inputs(self):
             return DefaultInputs(
@@ -108,9 +96,6 @@ class TestCustomOp:
             "description": "Custom Sparse MatMul Layer",
         }
 
-        def __init__(self, **kwargs):
-            super(TestCustomOp.custom_torch_sparse_matmul, self).__init__(**kwargs)
-
         def type_inference(self):
             x_type = self.x.dtype
             x_shape = self.x.shape
