@@ -4,24 +4,19 @@
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
 from collections import defaultdict
-import copy
 import logging
 import numbers
+
 import numpy as np
 
 from coremltools.converters.mil.mil.types.symbolic import any_symbolic
 from .program import Program, Placeholder
 from .block import curr_block, Function
 from .input_type import (
-    _InputType,
-    InternalStringInputType,
-    InternalScalarOrTensorInputType,
-    ScalarOrTensorInputType,
-    ListOrScalarOrTensorInputType,
-    TupleInputType,
-    InputSpec,
     InternalInputType,
-    PyFunctionInputType,
+    ListOrTensorInputType,
+    TensorInputType,
+    TupleInputType,
 )
 from .var import InternalVar, Var
 
@@ -51,10 +46,10 @@ class Builder:
     >>> from coremltools.converters.mil.mil import Program, Function
 
     >>> prog = Program()
-    >>> func_inputs = {"x": mb.placeholder(_shape=[2,3]),
-    >>>                "y": mb.placeholder(_shape=[2,3])}
+    >>> func_inputs = {"x": mb.placeholder(shape=[2,3]),
+    >>>                "y": mb.placeholder(shape=[2,3])}
     >>> with Function(func_inputs) as ssa_fun:
-    >>>   x, y = ssa_fun.inputs['x'], ssa_fun.inputs['x']
+    >>>   x, y = ssa_fun.inputs['x'], ssa_fun.inputs['y']
     >>>   res_var = mb.add(x=x, y=y) # created within ssa_fun block
     >>>   ssa_fun.set_outputs([res_var])
     >>> prog.add_function("main", ssa_fun)
@@ -149,8 +144,7 @@ class Builder:
                 update_dict[k] = var
                 continue
 
-            if isinstance(in_type, (ScalarOrTensorInputType,
-              ListOrScalarOrTensorInputType)):
+            if isinstance(in_type, (TensorInputType, ListOrTensorInputType)):
                 var = cls._add_const(val, new_var_name, before_op)
                 update_dict[k] = var
 
@@ -194,15 +188,15 @@ class Builder:
         return new_op.outputs
 
     @staticmethod
-    def placeholder(shape, dtype=None):
-        return Placeholder(shape, dtype)
+    def placeholder(shape, dtype=None, allow_rank0_input=False):
+        return Placeholder(shape, dtype, allow_rank0_input=allow_rank0_input)
 
     @staticmethod
     def TensorSpec(shape, dtype=None):
         return Placeholder(shape, dtype)
 
     @staticmethod
-    def program(input_specs=None):
+    def program(input_specs=None, opset_version=None):
         """
         
         The ``mb.program`` decorator creates a MIL program with a single 
@@ -212,13 +206,16 @@ class Builder:
         ----------
         
         input_specs: TensorSpec
-             Describes a tensor.
+            Describes a tensor.
+             
+        opset_version: AvailableTarget enum
+            Describes the opset version of the program
         
         
         Examples
         --------
-
-        >>> @mb.program(input_specs=[mb.TensorSpec(shape=(1,2))])
+        >>> import coremltools as ct
+        >>> @mb.program(input_specs=[mb.TensorSpec(shape=(1,2))], opset_version=ct.target.iOS16)
         >>> def prog(a):
         >>>     return mb.add(x=a, y=2)
 
@@ -238,7 +235,7 @@ class Builder:
                     )
                 )
             input_spec_dict = {k: v for k, v in zip(arg_names, input_specs)}
-            with Function(input_spec_dict) as func:
+            with Function(input_spec_dict, opset_version) as func:
                 input_vars = [func.inputs[a] for a in arg_names]
                 outputs = main_block(*input_vars)
                 if isinstance(outputs, tuple):
@@ -250,7 +247,3 @@ class Builder:
             return program
 
         return wrapper
-
-
-# importing ops triggers installation of all ops into Builder
-from .ops import defs as _ops
