@@ -3,11 +3,24 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-import scipy
-from coremltools.converters.mil import testing_reqs
-from coremltools.converters.mil.testing_reqs import *
+import itertools
 
+import numpy as np
+import pytest
+import scipy
+from scipy import special
 from .testing_utils import run_compare_builder
+from coremltools.converters.mil import testing_reqs
+from coremltools.converters.mil.mil import (
+    Builder as mb,
+    Function,
+    get_new_symbol,
+    Program,
+    types,
+)
+from coremltools.converters.mil.mil.types.symbolic import is_compatible_symbolic_vector
+from coremltools.converters.mil.testing_utils import ssa_fn
+
 
 backends = testing_reqs.backends
 
@@ -15,7 +28,7 @@ backends = testing_reqs.backends
 class TestElementwiseUnary:
     # All ops in this test share the same backends
     @pytest.mark.parametrize(
-        "use_cpu_only, backend, mode",
+        "use_cpu_for_conversion, backend, mode",
         itertools.product(
             [True, False],
             backends,
@@ -25,13 +38,13 @@ class TestElementwiseUnary:
                 "asin",
                 "atan",
                 "atanh",
-                "exp2",
+                "cast",
                 "clip",
                 "cos",
                 "cosh",
                 "erf",
                 "exp",
-                "erf",
+                "exp2",
                 "floor",
                 "inverse",
                 "log",
@@ -45,11 +58,10 @@ class TestElementwiseUnary:
                 "tan",
                 "tanh",
                 "threshold",
-                "cast",
             ],
         ),
     )
-    def test_builder_to_backend_smoke(self, use_cpu_only, backend, mode):
+    def test_builder_to_backend_smoke(self, use_cpu_for_conversion, backend, mode):
         if mode == "abs":
             val = np.array([[-1, 2, -3], [4, -5, 6]], dtype=np.float32)
             expected_outputs = np.array([[1, 2, 3], [4, 5, 6]], dtype=np.float32)
@@ -267,7 +279,7 @@ class TestElementwiseUnary:
             input_values,
             expected_output_types,
             expected_outputs,
-            use_cpu_only=use_cpu_only,
+            use_cpu_only=use_cpu_for_conversion,
             frontend_only=False,
             backend=backend,
         )
@@ -278,7 +290,7 @@ class TestElementwiseUnary:
         v = mb.abs(x=val)
         expected_outputs = np.array([[1, 2, 3], [4, 5, 6]], dtype=np.float32)
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_acos_eval(self):
@@ -288,7 +300,7 @@ class TestElementwiseUnary:
             [[3.14159265, 2.0943951, 1.57079633], [1.15927948, 1.04719755, 0.64350111]],
             dtype=np.float32,
         )
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_asin_eval(self):
@@ -299,7 +311,7 @@ class TestElementwiseUnary:
             dtype=np.float32,
         )
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_atan_eval(self):
@@ -313,7 +325,7 @@ class TestElementwiseUnary:
             dtype=np.float32,
         )
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_atanh_eval(self):
@@ -324,7 +336,7 @@ class TestElementwiseUnary:
             dtype=np.float32,
         )
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_cast_eval(self):
@@ -333,7 +345,7 @@ class TestElementwiseUnary:
 
         v = mb.cast(x=val, dtype="int32")
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_ceil_eval(self):
@@ -341,7 +353,7 @@ class TestElementwiseUnary:
         v = mb.ceil(x=val)
         expected_outputs = np.array([[-1, 2, -3], [5, -5, 7]], dtype=np.float32)
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_clip_eval(self):
@@ -349,7 +361,7 @@ class TestElementwiseUnary:
         v = mb.clip(x=val, alpha=0.0, beta=5.0)
         expected_outputs = np.array([[0, 2, 0], [4.5, 0, 5]], dtype=np.float32)
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_cos_eval(self):
@@ -363,7 +375,7 @@ class TestElementwiseUnary:
             dtype=np.float32,
         )
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_cosh_eval(self):
@@ -374,13 +386,13 @@ class TestElementwiseUnary:
             dtype=np.float32,
         )
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_erf_eval(self):
         x_val = np.array([[-1, 2, -3], [4, -5, 6]], dtype=np.float32)
         v = mb.erf(x=x_val)
-        assert is_close(scipy.special.erf(x_val), v.val)
+        np.testing.assert_allclose(scipy.special.erf(x_val), v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_exp_eval(self):
@@ -391,7 +403,7 @@ class TestElementwiseUnary:
             dtype=np.float32,
         )
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_exp2_eval(self):
@@ -401,7 +413,7 @@ class TestElementwiseUnary:
             [[0.5, 4.0, 0.125], [16, 0.03125, 64]], dtype=np.float32
         )
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_floor_eval(self):
@@ -409,7 +421,7 @@ class TestElementwiseUnary:
         v = mb.floor(x=val)
         expected_outputs = np.array([[-2, 2, -4], [4, -5, 6]], dtype=np.float32)
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_inverse_eval(self):
@@ -418,7 +430,7 @@ class TestElementwiseUnary:
         expected_outputs = np.array(
             [[-1.0, 0.5, -0.33333334], [0.25, -0.2, 0.16666667]], dtype=np.float32
         )
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_log_eval(self):
@@ -429,7 +441,7 @@ class TestElementwiseUnary:
             dtype=np.float32,
         )
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_round_eval(self):
@@ -437,7 +449,7 @@ class TestElementwiseUnary:
         v = mb.round(x=val)
         expected_outputs = np.array([[-1, 2, -3], [5, -5, 7]], dtype=np.float32)
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_rsqrt_eval(self):
@@ -448,7 +460,7 @@ class TestElementwiseUnary:
             dtype=np.float32,
         )
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_sign_eval(self):
@@ -456,7 +468,7 @@ class TestElementwiseUnary:
         v = mb.sign(x=val)
         expected_outputs = np.array([[-1, 1, 0], [0, -1, 1]], dtype=np.float32)
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_sin_eval(self):
@@ -470,7 +482,7 @@ class TestElementwiseUnary:
             dtype=np.float32,
         )
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_sinh_eval(self):
@@ -481,7 +493,7 @@ class TestElementwiseUnary:
             dtype=np.float32,
         )
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_sqrt_eval(self):
@@ -492,7 +504,7 @@ class TestElementwiseUnary:
             dtype=np.float32,
         )
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_tan_eval(self):
@@ -502,13 +514,13 @@ class TestElementwiseUnary:
             [[-1.5574, -2.185, 0.1425], [1.15782, 3.3805, -0.291]], dtype=np.float32
         )
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_tanh_eval(self):
         x_val = np.array([[-1, 2, -3], [4, -5, 6]], dtype=np.float32)
         v = mb.tanh(x=x_val)
-        assert is_close(np.tanh(x_val), v.val)
+        np.testing.assert_allclose(np.tanh(x_val), v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_threshold_eval(self):
@@ -516,7 +528,21 @@ class TestElementwiseUnary:
         v = mb.threshold(x=val, alpha=1.0)
         expected_outputs = np.array([[1.0, 2, 1.0], [4.5, 1.0, 6.7]], dtype=np.float32)
 
-        assert is_close(expected_outputs, v.val)
+        np.testing.assert_allclose(expected_outputs, v.val, atol=1e-04, rtol=1e-05)
+
+    def test_cast_with_symbolic_value(self):
+        input_shape = [get_new_symbol(), 1]
+        input_placeholders = {
+            "x": mb.placeholder(shape=input_shape),
+        }
+
+        def build(x):
+            shape = mb.shape(x=x)
+            return mb.cast(x=shape, dtype="int32")
+
+        with Function(input_placeholders) as ssa_func:
+            output_vars = build(**ssa_func.inputs)
+            assert is_compatible_symbolic_vector(output_vars.sym_val, [get_new_symbol(), 1])
 
     @pytest.mark.parametrize(
         "use_cpu_only, backend, epsilon",
@@ -613,3 +639,57 @@ class TestElementwiseUnary:
             frontend_only=False,
             backend=backend,
         )
+
+    @pytest.mark.parametrize(
+        "use_cpu_for_conversion, backend, src_dst",
+        itertools.product(
+            [True, False],
+            backends,
+            [("fp16", "fp32"), ("fp32", "fp16")],
+        ),
+    )
+    def test_builder_to_backend_stress_cast(
+            self, use_cpu_for_conversion, backend, src_dst
+    ):
+        src_dtype, dst_dtype = src_dst
+        x = np.array([[1, 2, 3], [4, 5, 6]], dtype=np.float32)
+        numpy_pred = x.astype(dtype=np.float16)
+
+        input_placeholder_dict = {"x": mb.placeholder(shape=x.shape)}
+        input_value_dict = {"x": x}
+
+        def build(x):
+            x = mb.cast(x=x, dtype=src_dtype)
+            x = mb.square(x=x)
+            x = mb.cast(x=x, dtype=dst_dtype)
+            x = mb.sqrt(x=x)
+            x = mb.cast(x=x, dtype="fp32")
+            return x
+
+        expected_output_type = x.shape + (types.fp32,)
+        run_compare_builder(
+            build,
+            input_placeholder_dict,
+            input_value_dict,
+            expected_output_type,
+            numpy_pred,
+            use_cpu_only=use_cpu_for_conversion,
+            frontend_only=False,
+            backend=backend,
+        )
+
+    def test_erf_value_inference(self):
+        INPUT_SIZE=(2, 3, 4)
+        rs = np.random.RandomState(1234)
+        x = rs.random(INPUT_SIZE)
+
+        @mb.program(input_specs=[])
+        def prog():
+            return mb.erf(x=x)
+
+        ops = list(prog.functions.values())[0].operations
+        assert len(ops) == 2
+        assert ops[0].op_type == 'const'
+        erf_op = ops[1]
+        assert erf_op.op_type == 'erf'
+        np.testing.assert_allclose(erf_op.value_inference(), scipy.special.erf(x), atol=1e-04, rtol=1e-05)
