@@ -5,10 +5,13 @@
 
 import functools
 import itertools
+
 import numpy as np
 import pytest
 
 from .testing_utils import run_compare_builder
+
+import coremltools as ct
 from coremltools.converters.mil import testing_reqs
 from coremltools.converters.mil.mil import (
     Builder as mb,
@@ -17,6 +20,7 @@ from coremltools.converters.mil.mil import (
 )
 from coremltools.converters.mil.testing_reqs import backends
 from coremltools.converters.mil.testing_utils import random_gen
+from coremltools.models.utils import _macos_version
 
 if testing_reqs._HAS_TORCH:
     import torch
@@ -28,7 +32,7 @@ class TestAffine:
     )
     def test_builder_to_backend_smoke(self, use_cpu_only, backend):
         if backend[0] == "neuralnetwork":
-            pytest.xfail("nn backend not supported")
+            pytest.skip("nn backend not supported")
 
         x_val = np.array([11.0, 22.0, 33.0, 44.0], dtype=np.float32).reshape(
             [1, 1, 2, 2]
@@ -97,11 +101,18 @@ class TestAffine:
 
 class TestResample:
     @pytest.mark.parametrize(
-        "use_cpu_only, backend", itertools.product([True, False], backends,)
+        "use_cpu_only, backend, minimum_deployment_target",
+        itertools.product(
+            [True, False],
+            backends,
+            [ct.target.iOS15, ct.target.iOS16],
+        )
     )
-    def test_builder_to_backend_smoke(self, use_cpu_only, backend):
+    def test_builder_to_backend_smoke(self, use_cpu_only, backend, minimum_deployment_target):
         if backend[0] == "neuralnetwork":
-            pytest.xfail("nn backend not supported")
+            pytest.skip("nn backend not supported")
+        if minimum_deployment_target == ct.target.iOS16 and _macos_version() < (13, 0):
+            pytest.skip("New functionality in macOS13/iOS16")
 
         x_ = np.array([11.0, 22.0, 33.0, 44.0], dtype=np.float32).reshape([1, 1, 2, 2])
         coordinates_ = np.array(
@@ -184,7 +195,7 @@ class TestResample:
                 expected_output_3,
             ],
         ):
-            run_compare_builder(
+            mlmodel = run_compare_builder(
                 build,
                 input_placeholder_dict,
                 input_value_dict,
@@ -192,7 +203,17 @@ class TestResample:
                 expected_output,
                 use_cpu_only=use_cpu_only,
                 backend=backend,
+                minimum_deployment_target=minimum_deployment_target,
             )
+            prog = mlmodel._mil_program
+            number_of_cast = len(prog["main"].find_ops(op_type="cast"))
+            # for the new iOS16 resample op, the coordinates is cast to fp16
+            if minimum_deployment_target == ct.target.iOS15:
+                assert number_of_cast == 2
+            elif minimum_deployment_target == ct.target.iOS16:
+                assert number_of_cast == 3
+            else:
+                raise ValueError("Unrecognized target {}".format(minimum_deployment_target))
 
 
 class TestResizeNearestNeighbor:
@@ -242,10 +263,10 @@ class TestUpsampleNearestNeighborFractionalScales:
     )
     def test_builder_to_backend_smoke(self, use_cpu_for_conversion, backend):
         if backend[0] == "neuralnetwork":
-            pytest.xfail("nn backend not supported")
-
+            pytest.skip("nn backend not supported")
+        
         if backend[0] == "mlprogram" and not use_cpu_for_conversion:
-            pytest.xfail("rdar://78343225 ((MIL GPU) Core ML Tools Unit Test failures [numerical error])")
+            pytest.xfail("rdar://97398448 (TestUpsampleNearestNeighborFractionalScales failing on GPU)")
 
         x_val = np.array([1.5, -2.5, 3.5], dtype=np.float32).reshape([1, 1, 1, 3])
         input_placeholder_dict = {"x": mb.placeholder(shape=x_val.shape)}
@@ -287,7 +308,6 @@ class TestUpsampleNearestNeighborFractionalScales:
             expected_outputs,
             use_cpu_only=use_cpu_for_conversion,
             backend=backend,
-            use_cpu_for_conversion=use_cpu_for_conversion,
         )
 
 
@@ -299,6 +319,9 @@ class TestResizeBilinear:
         if backend[0] == "mlprogram":
             pytest.xfail("Seg fault: rdar://78343191 ((MIL GPU) Core ML Tools Unit Test failures [failure to load or Seg fault])")
 
+        if backend[0] == "neuralnetwork" and use_cpu_only:
+            pytest.xfail("rdar://85318710 (Coremltools Smoke test on ResizeBilinear failing on NNv1 backend.)")
+
         x = np.array([0, 1], dtype=np.float32).reshape(1, 1, 2)
         input_placeholder_dict = {"x": mb.placeholder(shape=x.shape)}
         input_value_dict = {"x": x}
@@ -400,9 +423,6 @@ class TestUpsampleBilinear:
         "use_cpu_only, backend", itertools.product([True, False], backends,)
     )
     def test_builder_to_backend_smoke(self, use_cpu_only, backend):
-        if backend[0] == "mlprogram" and not use_cpu_only:
-            pytest.xfail("test failing on gpu with nan output")
-
         x = np.array([0, 1], dtype=np.float32).reshape(1, 1, 2)
         input_placeholder_dict = {"x": mb.placeholder(shape=x.shape)}
         input_value_dict = {"x": x}
@@ -449,6 +469,60 @@ class TestUpsampleBilinear:
             backend=backend,
         )
 
+    @pytest.mark.parametrize(
+        "use_cpu_only, backend, align_corners, half_pixel_centers", 
+        itertools.product(
+            [True, False], 
+            backends,
+            [True, False],
+            [True, False],
+        )
+    )
+    def test_builder_to_backend_smoke_iOS16(self, use_cpu_only, backend, align_corners, half_pixel_centers):
+        if backend[0] == "neuralnetwork" or ct.utils._macos_version() < (13, 0):
+            pytest.skip("The new half_pixel_centers argument only available in iOS16")
+
+        if align_corners and half_pixel_centers:
+            pytest.skip("Invalid configuration of align_corners and half_pixel_centers")
+
+        x = np.array([1, 2], dtype=np.float32).reshape(1, 1, 1, 2)
+        input_placeholder_dict = {"x": mb.placeholder(shape=x.shape)}
+        input_value_dict = {"x": x}
+
+        def build_upsample_bilinear(x):
+            return mb.upsample_bilinear(
+                x=x,
+                scale_factor_height=2,
+                scale_factor_width=3,
+                align_corners=align_corners,
+                half_pixel_centers=half_pixel_centers,
+            )
+
+        expected_output_type = (1, 1, 2, 6, types.fp32)
+        
+        if align_corners and not half_pixel_centers:
+            expected_output = [1., 1.2, 1.4, 1.6, 1.8, 2., 1., 1.2, 1.4, 1.6, 1.8, 2.]
+        elif not align_corners and half_pixel_centers:
+            expected_output = [1., 1., 1.33334, 1.66667, 2., 2., 1., 1., 1.33334, 1.66667, 2., 2.]
+        elif not align_corners and not half_pixel_centers:
+            expected_output = [1., 1.33334, 1.66667, 2., 2., 2., 1., 1.33334, 1.66667, 2., 2., 2.]
+        else:
+            raise ValueError("align_corners and half_pixel_centers cannot be both True")
+
+        expected_output = [np.array(expected_output, dtype=np.float32).reshape(1, 1, 2, 6)]
+
+        run_compare_builder(
+            build_upsample_bilinear,
+            input_placeholder_dict,
+            input_value_dict,
+            expected_output_type,
+            expected_output,
+            use_cpu_only=use_cpu_only,
+            frontend_only=False,
+            backend=backend,
+            minimum_deployment_target=ct.target.iOS16,
+        )
+
     @pytest.mark.skipif(not testing_reqs._HAS_TORCH, reason="PyTorch not installed.")
     @pytest.mark.parametrize(
         "use_cpu_only, backend, input_shape, scale_factor, align_corners, recompute_scale_factor",
@@ -641,15 +715,70 @@ class TestCrop:
 
 
 class TestCropResize:
+    @pytest.mark.parametrize(
+        "use_cpu_only, backend",
+        itertools.product([True, False], backends),
+    )
+    def test_builder_to_backend_smoke_pad_value(self, use_cpu_only, backend):
+        if backend[0] == "neuralnetwork":
+            pytest.skip("pad_mode only supported on iOS16 or above")
+            
+        if ct.utils._macos_version() < (13, 0):
+            pytest.skip("pad_value not supported in macOS12 or older.")
+
+        x = np.array(
+            [[1, 2, 3, 4], [5, 6, 7, 8], [9, 10, 11, 12], [13, 14, 15, 16]],
+            dtype=np.float32,
+        ).reshape(1, 1, 4, 4)
+
+        roi = np.array([
+            [0, 0.1, 0.3, 1.3, 1],
+            [0, 0.5, 1.8, 1., 0.3],
+            [0, 0.0, 0.4, 0.6, 0.7],
+        ], dtype=np.float32).reshape(3, 1, 5, 1, 1)
+        
+        def build(x):
+            return mb.crop_resize(
+                    x=x,
+                    roi=roi,
+                    target_width=2,
+                    target_height=2,
+                    normalized_coordinates=True,
+                    box_coordinate_mode="CORNERS_HEIGHT_FIRST",
+                    sampling_mode="ALIGN_CORNERS",
+                    pad_value=10.0,
+            )
+        
+        expected_output_type = [
+            (3, 1, 1, 2, 2, types.fp32),
+        ]
+        expected_output = [
+            np.array([ 3.1, 5.2, 10, 10, 10, 7.899, 10, 13.9, 2.2, 3.1, 9.4, 10.3], dtype=np.float32).reshape(3, 1, 1, 2, 2),
+        ]
+
+        input_placeholder_dict = {"x": mb.placeholder(shape=(1, 1, 4, 4))}
+        input_value_dict = {"x": x}
+
+        run_compare_builder(
+            build,
+            input_placeholder_dict,
+            input_value_dict,
+            expected_output_type,
+            expected_output,
+            use_cpu_only=use_cpu_only,
+            frontend_only=False,
+            backend=backend,
+            minimum_deployment_target=ct.target.iOS16,
+        )
+        
+        
     @pytest.mark.parametrize(
         "use_cpu_only, backend, is_symbolic",
         itertools.product([True, False], backends, [True, False]),
     )
     def test_builder_to_backend_smoke(self, use_cpu_only, backend, is_symbolic):
-
-        if backend[0] == "mlprogram":
-            pytest.xfail("rdar://78343191 ((MIL GPU) Core ML Tools Unit Test failures [failure to load or Seg fault])")
-
+        if backend[0] == "mlprogram" and not use_cpu_only:
+            pytest.xfail("rdar://97398582 (TestCropResize failing on mlprogram + GPU)")
         x = np.array(
             [[1, 2, 3, 4], [5, 6, 7, 8], [9, 10, 11, 12], [13, 14, 15, 16]],
             dtype=np.float32,
