@@ -3,12 +3,22 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from coremltools.converters.mil import testing_reqs
-from coremltools.converters.mil.testing_reqs import *
+import itertools
+
+import numpy as np
+import pytest
 
 from .testing_utils import run_compare_builder
 
-backends = testing_reqs.backends
+import coremltools as ct
+from coremltools._deps import _HAS_TF_1, MSG_TF1_NOT_FOUND
+from coremltools.converters.mil import testing_reqs
+from coremltools.converters.mil.mil import Builder as mb, types
+from coremltools.converters.mil.testing_reqs import backends
+from coremltools.converters.mil.testing_utils import ssa_fn
+
+if _HAS_TF_1:
+    import tensorflow as tf
 
 
 class TestScatter:
@@ -392,32 +402,101 @@ class TestGather:
             backend=backend,
         )
 
+
     @pytest.mark.parametrize(
         "use_cpu_only, backend", itertools.product([True, False], backends,)
     )
-    def test_embedding_builder_to_backend_smoke(self, use_cpu_only, backend):
-        x = np.array([[1, 2, 3], [4, 5, 6]], dtype=np.float32)
-        indices = np.array([1, 0], dtype=np.int32)
+    def test_builder_to_backend_smoke_iOS16(self, use_cpu_only, backend):
+        if backend[0] == "neuralnetwork":
+            pytest.skip("nn backend not supported")
+        if ct.utils._macos_version() < (13, 0):
+            pytest.skip("batch_dims not supported in macOS12 or older.")
+
+        x = np.array([[[1, 2, 3], [4, 5, 6]], [[7, 8, 9], [10, 11, 12]]], dtype=np.float32)
+        indices = np.array([[[1, 0], [0, 1]], [[1, 0], [0, 0]]], dtype=np.int32)
+
         input_placeholders = {
+            "x": mb.placeholder(shape=x.shape),
             "indices": mb.placeholder(shape=indices.shape, dtype=types.int32),
         }
 
-        input_values = {"indices": indices}
+        input_values = {"x": x, "indices": indices}
 
-        def build(indices):
+        def build(x, indices):
             return [
-                mb.gather(x=x, indices=indices, axis=0),
-                mb.gather(x=x, indices=indices, axis=-2),
+                mb.gather(x=x, indices=indices, axis=1, batch_dims=0),
+                mb.gather(x=x, indices=indices, axis=1, batch_dims=1),
+                mb.gather(x=x, indices=indices, axis=2, batch_dims=0),
+                mb.gather(x=x, indices=indices, axis=2, batch_dims=1),
+                mb.gather(x=x, indices=indices, axis=2, batch_dims=2),
             ]
 
         expected_output_types = [
-            (2, 3, types.fp32),
-            (2, 3, types.fp32),
+            (2, 2, 2, 2, 3, types.fp32),
+            (2, 2, 2, 3, types.fp32),
+            (2, 2, 2, 2, 2, types.fp32),
+            (2, 2, 2, 2, types.fp32),
+            (2, 2, 2, types.fp32),
         ]
 
         expected_outputs = [
-            np.array([[4, 5, 6], [1, 2, 3]], dtype=np.float32),
-            np.array([[4, 5, 6], [1, 2, 3]], dtype=np.float32),
+            np.array([[[[[ 4,  5,  6],
+                         [ 1,  2,  3]],
+                        [[ 1,  2,  3],
+                         [ 4,  5,  6]]],
+                       [[[ 4,  5,  6],
+                         [ 1,  2,  3]],
+                        [[ 1,  2,  3],
+                         [ 1,  2,  3]]]],
+                      [[[[10, 11, 12],
+                         [ 7,  8,  9]],
+                        [[ 7,  8,  9],
+                         [10, 11, 12]]],
+                       [[[10, 11, 12],
+                         [ 7,  8,  9]],
+                        [[ 7,  8,  9],
+                         [ 7,  8,  9]]]]], dtype=np.float32
+            ), 
+            np.array([[[[ 4,  5,  6],
+                        [ 1,  2,  3]],
+                       [[ 1,  2,  3],
+                        [ 4,  5,  6]]],
+                      [[[10, 11, 12],
+                        [ 7,  8,  9]],
+                       [[ 7,  8,  9],
+                        [ 7,  8,  9]]]], dtype=np.float32
+            ),
+            np.array([[[[[ 2,  1],
+                         [ 1,  2]],
+                        [[ 2,  1],
+                         [ 1,  1]]],
+                       [[[ 5,  4],
+                         [ 4,  5]],
+                        [[ 5,  4],
+                         [ 4,  4]]]],
+                      [[[[ 8,  7],
+                         [ 7,  8]],
+                        [[ 8,  7],
+                         [ 7,  7]]],
+                       [[[11, 10],
+                         [10, 11]],
+                        [[11, 10],
+                         [10, 10]]]]], dtype=np.float32
+            ),
+            np.array([[[[ 2,  1],
+                        [ 1,  2]],
+                       [[ 5,  4],
+                        [ 4,  5]]],
+                      [[[ 8,  7],
+                        [ 7,  7]],
+                       [[11, 10],
+                        [10, 10]]]], dtype=np.float32
+            ),
+            np.array([[[ 2,  1],
+                       [ 4,  5]],
+                      [[ 8,  7],
+                       [10, 10]]], dtype=np.float32
+            ),
         ]
 
         run_compare_builder(
@@ -427,8 +506,27 @@ class TestGather:
             expected_output_types,
             expected_outputs,
             use_cpu_only=use_cpu_only,
-            frontend_only=False,
             backend=backend,
+            minimum_deployment_target=ct.target.iOS16,
+        )
+
+
+    def test_builder_eval_iOS16(self):
+        @mb.program(input_specs=[mb.TensorSpec(shape=(1, ), dtype=types.fp32)], opset_version=ct.target.iOS16)
+        def prog(x):
+            params = np.array([[[1, 2, 3], [4, 5, 6]], [[7, 8, 9], [10, 11, 12]]], dtype=np.float32)
+            indices = np.array([[[1, 0], [0, 1]], [[1, 0], [0, 0]]], dtype=np.int32)
+            res = mb.gather(x=params, indices=indices, axis=2, batch_dims=2)
+            return res
+
+        main_func = prog.functions["main"]
+        gather_ops = main_func.find_ops(op_type="gather")[0]
+
+        np.testing.assert_allclose(
+            np.array([[[ 2,  1], [ 4,  5]], [[ 8,  7], [10, 10]]], dtype=np.float32), 
+            gather_ops.outputs[0].val, 
+            atol=1e-04, 
+            rtol=1e-05
         )
 
     @pytest.mark.parametrize(
@@ -483,10 +581,6 @@ class TestGatherAlongAxis:
         "use_cpu_only, backend", itertools.product([True, False], backends,)
     )
     def test_builder_to_backend_smoke(self, use_cpu_only, backend):
-
-        if backend[0] == "mlprogram" and not use_cpu_only:
-            pytest.xfail("rdar://80710279 (Crash in GatherAlongAxis unit test on MIL GPU context)")
-
         x = np.array([[1, 2, 3], [4, 5, 6]], dtype=np.float32)
         indices = np.array([[1, 0, 1], [1, 1, 0]], dtype=np.int32)
         input_placeholders = {
@@ -549,8 +643,7 @@ class TestGatherAlongAxis:
     )
     def test_builder_to_backend_programmatic(self, use_cpu_for_conversion, backend, rank_axis):
         if backend[0] == "mlprogram" and not use_cpu_for_conversion:
-            pytest.xfail("rdar://78343225 ((MIL GPU) Core ML Tools Unit Test failures [numerical error])")
-
+            pytest.xfail("rdar://97398875 (TestGatherAlongAxis failing on mlprgram + GPU)")
         rank, axis = rank_axis
         x_shape = np.random.randint(low=2, high=8, size=rank)
         indices_shape = np.copy(x_shape)
@@ -583,7 +676,6 @@ class TestGatherAlongAxis:
             use_cpu_only=use_cpu_for_conversion,
             frontend_only=False,
             backend=backend,
-            use_cpu_for_conversion=use_cpu_for_conversion,
         )
 
 
@@ -617,3 +709,56 @@ class TestGatherNd:
             frontend_only=False,
             backend=backend,
         )
+
+    @pytest.mark.parametrize(
+        "use_cpu_only, backend", itertools.product([True, False], backends,)
+    )
+    def test_builder_to_backend_smoke_iOS16(self, use_cpu_only, backend):
+        if backend[0] == "neuralnetwork":
+            pytest.skip("nn backend not supported")
+
+        if ct.utils._macos_version() < (13, 0):
+            pytest.skip("batch_dims not supported in macOS12 or older.")
+
+        x = np.array([[[1, 2, 3], [4, 5, 6]], [[7, 8, 9], [10, 11, 12]]], dtype=np.float32)
+        indices = np.array([[[1, 0], [0, 1]], [[1, 0], [0, 0]]], dtype=np.int32)
+
+        input_placeholders = {
+            "x": mb.placeholder(shape=x.shape),
+            "indices": mb.placeholder(shape=indices.shape, dtype=types.int32),
+        }
+
+        input_values = {"x": x, "indices": indices}
+
+        def build(x, indices):
+            return [
+                mb.gather_nd(x=x, indices=indices, batch_dims=0),
+                mb.gather_nd(x=x, indices=indices, batch_dims=1),
+            ]
+
+        expected_output_types = [
+            (2, 2, 3, types.fp32),
+            (2, 2, types.fp32)
+        ]
+
+        expected_outputs = [
+            np.array([[[7, 8, 9],
+                       [4, 5, 6]],
+                      [[7, 8, 9],
+                       [1, 2, 3]]], dtype=np.float32
+            ),
+            np.array([[ 4,  2],
+                      [10,  7]], dtype=np.float32
+            ),
+        ]
+
+        run_compare_builder(
+            build,
+            input_placeholders,
+            input_values,
+            expected_output_types,
+            expected_outputs,
+            use_cpu_only=use_cpu_only,
+            backend=backend,
+            minimum_deployment_target=ct.target.iOS16,
+        )
