@@ -2,10 +2,13 @@
 #
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
+
 import itertools
+
 import numpy as np
 import pytest
 
+import coremltools as ct
 from coremltools.converters.mil import testing_reqs
 from coremltools.converters.mil.mil import (
     Builder as mb,
@@ -13,6 +16,7 @@ from coremltools.converters.mil.mil import (
     types
 )
 from coremltools.converters.mil.testing_utils import random_gen, ssa_fn, get_op_types_in_program
+from coremltools.models.utils import _macos_version
 from .testing_utils import UNK_SYM, UNK_VARIADIC, run_compare_builder
 
 if testing_reqs._HAS_TF_1 or testing_reqs._HAS_TF_2:
@@ -170,45 +174,79 @@ class TestCumSum:
     def test_invalid_reverse2(self):
         x_val = random_gen(shape=(1, 2, 3, 4, 5), rand_min=-100, rand_max=100)
         with pytest.raises(ValueError):
-            pred = mb.cumsum(x=x_val, reverse=0)
+            mb.cumsum(x=x_val, reverse=0)
 
     @ssa_fn
     def test_invalid_reverse3(self):
         x_val = random_gen(shape=(1, 2, 3, 4, 5), rand_min=-100, rand_max=100)
         with pytest.raises(ValueError):
-            pred = mb.cumsum(x=x_val, reverse=1)
+            mb.cumsum(x=x_val, reverse=1)
 
     @ssa_fn
     def test_invalid_exclusive1(self):
         x_val = random_gen(shape=(1, 2, 3, 4, 5), rand_min=-100, rand_max=100)
         with pytest.raises(ValueError):
-            pred = mb.cumsum(x=x_val, exclusive="")
+            mb.cumsum(x=x_val, exclusive="")
 
     @ssa_fn
     def test_invalid_exclusive2(self):
         x_val = random_gen(shape=(1, 2, 3, 4, 5), rand_min=-100, rand_max=100)
         with pytest.raises(ValueError):
-            pred = mb.cumsum(x=x_val, exclusive=0)
+            mb.cumsum(x=x_val, exclusive=0)
 
     @ssa_fn
     def test_invalid_exclusive3(self):
         x_val = random_gen(shape=(1, 2, 3, 4, 5), rand_min=-100, rand_max=100)
         with pytest.raises(ValueError):
-            pred = mb.cumsum(x=x_val, exclusive=1)
+            mb.cumsum(x=x_val, exclusive=1)
 
     @ssa_fn
     def test_invalid_input1(self):
         x_val = 1
         with pytest.raises(ValueError):
-            pred = mb.cumsum(x=x_val)
+            mb.cumsum(x=x_val)
 
     @ssa_fn
     def test_invalid_input2(self):
         x_val = ["1"]
         with pytest.raises(ValueError):
-            pred = mb.cumsum(x=x_val)
+            mb.cumsum(x=x_val)
 
 
+class TestFillLike:
+    @pytest.mark.parametrize(
+        "use_cpu_only, backend", itertools.product([True, False], backends)
+    )
+    def test_builder_to_backend_smoke(self, use_cpu_only, backend):
+        if backend[0] == "neuralnetwork":
+            pytest.xfail("nn backend not supported")
+            
+        if ct.utils._macos_version() < (13, 0):
+            pytest.skip("fill_like not supported in macOS12 or older.")
+
+        shape = (2, 1, 3)
+        x_val = np.zeros(shape=shape, dtype=np.float32)
+        input_placeholders = {"x": mb.placeholder(shape=x_val.shape, dtype=types.int32)}
+
+        input_values = {"x": x_val}
+
+        def build(x):
+            return mb.fill_like(ref_tensor=x, value=1.0)
+
+        expected_output_types = [(2, 1, 3, types.fp32)]
+        expected_outputs = [np.full(shape=shape, fill_value=1.0)]
+
+        mlmodel = run_compare_builder(
+            build,
+            input_placeholders,
+            input_values,
+            expected_output_types,
+            expected_outputs,
+            use_cpu_only=use_cpu_only,
+            backend=backend,
+            minimum_deployment_target=ct.target.iOS16,
+        )
+
 class TestFill:
     @pytest.mark.parametrize(
         "use_cpu_only, backend", itertools.product([True, False], backends)
@@ -562,6 +600,7 @@ class TestNonMaximumSuppression:
             iou_threshold = np.maximum(np.min(iou_matrix) - 0.01, 0.0)
         else:
             iou_threshold = np.percentile(iou_matrix, iou_threshold_percentile) + 0.01
+        iou_threshold = np.maximum(iou_threshold, 1e-8)
 
         (
             tf_boxes,
@@ -984,7 +1023,6 @@ class TestRange1d:
         "use_cpu_only, backend", itertools.product([True, False], backends,)
     )
     def test_large_array(self, use_cpu_only, backend):
-        
         input_placeholders = {
             "x": mb.placeholder(shape=(1,)), # dummpy input
         }
@@ -1153,6 +1191,55 @@ class TestTopK:
             backend=backend,
         )
 
+    @pytest.mark.parametrize(
+        "use_cpu_only, backend, return_indices, sort", 
+        itertools.product(
+            [True, False], 
+            backends,
+            [True, False],
+            [True, False],
+        )
+    )
+    def test_builder_to_backend_smoke_iOS16(self, use_cpu_only, backend, return_indices, sort):
+        if backend[0] == "neuralnetwork":
+            pytest.skip("nn backend not supported")
+        if _macos_version() < (13, 0):
+            pytest.skip("New functionality in macOS13/iOS16")
+
+        if not return_indices:
+            pytest.xfail("rdar://92880117 (Topk with return_indices = False error out at the MIL->EIR stage)")
+
+        val = np.array([[-1.0, 2.0, -3.0], [4.0, -5.0, 6.0]], dtype=np.float32)
+        input_placeholders = {"x": mb.placeholder(shape=val.shape)}
+        input_values = {"x": val}
+        
+        def build(x):
+            return mb.topk(x=x, k=2, axis=1, return_indices=return_indices, sort=sort)
+
+        expected_output_types = [
+            (2, 2, types.fp32),
+            (2, 2, types.int32),
+        ]
+        expected_outputs = [
+            np.array([[2.0, -1.0], [6.0, 4.0]], dtype=np.float32),
+            np.array([[1, 0], [2, 0]], dtype=np.float32),
+        ]
+
+        if not return_indices:
+            expected_output_types = expected_output_types[:1]
+            expected_outputs = expected_outputs[:1]
+
+        run_compare_builder(
+            build,
+            input_placeholders,
+            input_values,
+            expected_output_types,
+            expected_outputs,
+            use_cpu_only=use_cpu_only,
+            backend=backend,
+            minimum_deployment_target=ct.target.iOS16,
+        )
+
     @ssa_fn
     def test_builder_eval(self):
         def np_topk(x, k, axis, ascending=False):
