@@ -3,13 +3,26 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from coremltools.converters.mil import testing_reqs
-from coremltools.converters.mil.mil import get_new_symbol, get_new_variadic_symbol
-from coremltools.converters.mil.testing_reqs import *
+import itertools
+
+import numpy as np
+import pytest
 
 from .testing_utils import UNK_SYM, UNK_VARIADIC, run_compare_builder
+import coremltools as ct
+from coremltools._deps import _HAS_TORCH
+from coremltools.converters.mil import testing_reqs
+from coremltools.converters.mil.mil import (
+    Builder as mb,
+    get_new_symbol,
+    types
+)
+from coremltools.converters.mil.mil.types import nptype_from_builtin
+from coremltools.converters.mil.testing_reqs import backends
+from coremltools.converters.mil.testing_utils import ssa_fn
 
-backends = testing_reqs.backends
+if _HAS_TORCH:
+    import torch
 
 
 class TestDepthToSpace:
@@ -40,6 +53,82 @@ class TestDepthToSpace:
         )
 
 
+class TestSpaceToBatch:
+    @pytest.mark.parametrize(
+        "use_cpu_only, backend", itertools.product([True, False], backends,)
+    )
+    def test_builder_to_backend_smoke(self, use_cpu_only, backend):
+        # original input type is (2, 1, 2, 4, fp32)
+        val = np.array([[[[ 1,  2,  3,  4],
+                          [ 5,  6,  7,  8]]],
+                        [[[ 9, 10, 11, 12],
+                          [13, 14, 15, 16]]]], dtype=np.float32)
+        input_placeholders = {"x": mb.placeholder(shape=val.shape)}
+        input_values = {"x": val}
+
+        def build(x):
+            return [mb.space_to_batch(x=x, block_shape=[2, 2], paddings=[[0, 0], [2, 0]])]
+
+        expected_output_types = (8, 1, 1, 3, types.fp32)
+        expected_outputs = np.array([[[[ 0,  1,  3]]],
+                                     [[[ 0,  9, 11]]],
+                                     [[[ 0,  2,  4]]],
+                                     [[[ 0, 10, 12]]],
+                                     [[[ 0,  5,  7]]],
+                                     [[[ 0, 13, 15]]],
+                                     [[[ 0,  6,  8]]],
+                                     [[[ 0, 14, 16]]]], dtype=np.float32)
+
+        run_compare_builder(
+            build,
+            input_placeholders,
+            input_values,
+            expected_output_types,
+            expected_outputs,
+            use_cpu_only=use_cpu_only,
+            frontend_only=False,
+            backend=backend,
+        )
+
+
+class TestBatchToSpace:
+    @pytest.mark.parametrize(
+        "use_cpu_only, backend", itertools.product([True, False], backends,)
+    )    
+    def test_builder_to_backend_smoke(self, use_cpu_only, backend):
+        # original input type is (8, 1, 1, 3, fp32)
+        val = np.array([[[[ 0,  1,  3]]],
+                       [[[ 0,  9, 11]]],
+                       [[[ 0,  2,  4]]],
+                       [[[ 0, 10, 12]]],
+                       [[[ 0,  5,  7]]],
+                       [[[ 0, 13, 15]]],
+                       [[[ 0,  6,  8]]],
+                       [[[ 0, 14, 16]]]], dtype=np.float32)
+        input_placeholders = {"x": mb.placeholder(shape=val.shape)}
+        input_values = {"x": val}
+
+        def build(x):
+            return [mb.batch_to_space(x=x, block_shape=[2, 2], crops=[[0, 0], [2, 0]])]
+
+        expected_output_types = (2, 1, 2, 4, types.fp32)
+        expected_outputs = np.array([[[[ 1,  2,  3,  4],
+                                       [ 5,  6,  7,  8]]],
+                                     [[[ 9, 10, 11, 12],
+                                       [13, 14, 15, 16]]]], dtype=np.float32)
+
+        run_compare_builder(
+            build,
+            input_placeholders,
+            input_values,
+            expected_output_types,
+            expected_outputs,
+            use_cpu_only=use_cpu_only,
+            frontend_only=False,
+            backend=backend,
+        )
+
+
 class TestExpandDims:
     @pytest.mark.parametrize(
         "use_cpu_only, backend", itertools.product([True, False], backends,)
@@ -130,17 +219,17 @@ class TestExpandDims:
     def test_builder_eval(self):
         x_val = np.random.rand(1, 6)
         v1 = mb.expand_dims(x=x_val, axes=[2])
-        assert is_close(np.expand_dims(x_val, 2), v1.val)
+        np.testing.assert_allclose(np.expand_dims(x_val, 2), v1.val, atol=1e-04, rtol=1e-05)
 
         v2 = mb.expand_dims(x=x_val, axes=[-1])
-        assert is_close(np.expand_dims(x_val, -1), v2.val)
+        np.testing.assert_allclose(np.expand_dims(x_val, -1), v2.val, atol=1e-04, rtol=1e-05)
 
         v3 = mb.expand_dims(x=x_val, axes=[-1, -2])
         ref = np.expand_dims(np.expand_dims(x_val, -1), -1)
-        assert is_close(ref, v3.val)
+        np.testing.assert_allclose(ref, v3.val, atol=1e-04, rtol=1e-05)
 
         v4 = mb.expand_dims(x=x_val, axes=[0, -1, -2])
-        assert is_close(np.reshape(x_val, (1, 1, 6, 1, 1)), v4.val)
+        np.testing.assert_allclose(np.reshape(x_val, (1, 1, 6, 1, 1)), v4.val, atol=1e-04, rtol=1e-05)
 
     @pytest.mark.parametrize(
         "use_cpu_only, backend, rank_and_axis",
@@ -231,6 +320,89 @@ class TestExpandDims:
             backend=backend,
         )
 
+class TestReshapeLike:
+    @pytest.mark.parametrize(
+        "use_cpu_only, backend, InputShape_RefShapes_Begins_Ends_EndMasks, InputType_RefType",
+        itertools.product(
+            [True, False],
+            backends,
+            [
+                [(4, 3), ((2, 2, 3), (1, 3)), (0, 1), (2, 2), (False, False)],
+                [(32,), ((1, 2, 2, 2), (3, 2, 2)), (1, 1), (0, 0), (True, True)],
+                [(72, 1), ((1, 2, 3, 4, 1), (3,)), (1, 0), (0, 1), (True, False)],
+            ],
+            [(types.bool, types.fp32), (types.fp32, types.bool)],
+        )
+    )
+    def test_builder_to_backend_smoke(
+            self,
+            use_cpu_only,
+            backend,
+            InputShape_RefShapes_Begins_Ends_EndMasks,
+            InputType_RefType,
+        ):
+        if backend[0] == "neuralnetwork":
+            pytest.skip("reshape_like not supoprted in neuralnetwork backend.")
+            
+        if ct.utils._macos_version() < (13, 0):
+            pytest.skip("reshape_like not supported in macOS12 or older.")
+            
+        input_shape, ref_shapes, begins, ends, end_masks = InputShape_RefShapes_Begins_Ends_EndMasks
+        ref_shape_1, ref_shape_2 = ref_shapes
+        input_type, ref_type = InputType_RefType
+        
+        t = np.random.rand(*input_shape).astype(np.float32)
+        ref_tensor_1 = np.random.rand(*ref_shape_1).astype(np.float32)
+        ref_tensor_2 = np.random.rand(*ref_shape_2).astype(np.float32)
+
+        input_placeholders = {
+            "x": mb.placeholder(shape=t.shape), 
+            "ref_tensor_1": mb.placeholder(shape=ref_shape_1),
+            "ref_tensor_2": mb.placeholder(shape=ref_shape_2),
+        }
+        input_values = {
+            "x": t,
+            "ref_tensor_1": ref_tensor_1,
+            "ref_tensor_2": ref_tensor_2,
+        }
+
+        def build(x, ref_tensor_1, ref_tensor_2):
+            if input_type == types.bool:
+                x = mb.cast(x=x, dtype="bool")
+                
+            if ref_type == types.bool:
+                ref_tensor_1 = mb.cast(x=ref_tensor_1, dtype="bool")
+                ref_tensor_2 = mb.cast(x=ref_tensor_2, dtype="bool")
+                
+            ref_tensors = (ref_tensor_1, ref_tensor_2)
+            return mb.reshape_like(x=x, ref_tensors=ref_tensors, begins=begins, ends=ends, end_masks=end_masks)
+            
+        output_shape = ()
+        for ref_shape, begin, end, end_mask in zip((ref_shape_1, ref_shape_2), begins, ends, end_masks):
+            if end_mask:
+                output_shape += tuple(ref_shape[begin:])
+            else:
+                output_shape += tuple(ref_shape[begin:end])
+
+        expected_output_types = [
+            output_shape + (input_type,),
+        ]
+        expected_outputs = [
+            np.reshape(t, output_shape).astype(nptype_from_builtin(input_type)),
+        ]
+
+        run_compare_builder(
+            build,
+            input_placeholders,
+            input_values,
+            expected_output_types,
+            expected_outputs,
+            use_cpu_only=use_cpu_only,
+            frontend_only=False,
+            backend=backend,
+            minimum_deployment_target=ct.target.iOS16
+        )
+
 
 class TestReshape:
     @pytest.mark.parametrize(
@@ -275,10 +447,10 @@ class TestReshape:
         t = np.array([[1, 2, 3], [4, 5, 6]], dtype=np.float32)
         r = mb.reshape(x=t, shape=[3, 2])
         expected_r = np.array([[1, 2], [3, 4], [5, 6]], dtype=np.float32)
-        assert is_close(expected_r, r.val)
+        np.testing.assert_allclose(expected_r, r.val, atol=1e-04, rtol=1e-05)
         r2 = mb.reshape(x=t, shape=[2, -1])
         expected_r2 = np.array([[1, 2, 3], [4, 5, 6]], dtype=np.float32)
-        assert is_close(expected_r2, r2.val)
+        np.testing.assert_allclose(expected_r2, r2.val, atol=1e-04, rtol=1e-05)
 
     @pytest.mark.parametrize(
         "use_cpu_only, backend", itertools.product([True, False], backends,)
@@ -286,13 +458,9 @@ class TestReshape:
     def test_builder_to_backend_symbolic(self, use_cpu_only, backend):
         s0 = get_new_symbol()
         s_len = get_new_symbol()
-        s1 = get_new_variadic_symbol()
 
-        # Test variadic (rdar://59559656)
         input_placeholders = {
             "x": mb.placeholder(shape=(2, s0)),
-            # TODO: variadic (rdar://59559656)
-            # "x2": mb.placeholder(shape=(s1, 2)),
             "shape": mb.placeholder(shape=(3,), dtype=types.int32),
             "shape2": mb.placeholder(shape=(s_len,), dtype=types.int32),
         }
@@ -302,8 +470,6 @@ class TestReshape:
                 mb.reshape(x=x, shape=[2, -1]),
                 mb.reshape(x=x, shape=[1, -1]),
                 mb.reshape(x=x, shape=[2, 1, 1, -1]),
-                # TODO: variadic (rdar://59559656)
-                # mb.reshape(x=x2, shape=[2, 1, 1]),
                 mb.reshape(x=x, shape=shape),
                 mb.reshape(x=x, shape=shape2),
             ]
@@ -312,8 +478,6 @@ class TestReshape:
             (2, s0, types.fp32),
             (1, 2 * s0, types.fp32),
             (2, 1, 1, s0, types.fp32),
-            # TODO: variadic (rdar://59559656)
-            # (2, 1, 1, types.fp32),
             (UNK_SYM, UNK_SYM, UNK_SYM, types.fp32),
             (UNK_VARIADIC, types.fp32),
         ]
@@ -321,17 +485,12 @@ class TestReshape:
             np.array([[1, 2, 3], [4, 5, 6]], dtype=np.float32),
             np.array([[1, 2, 3, 4, 5, 6]], dtype=np.float32),
             np.array([[[[1.0, 2.0, 3.0]]], [[[4.0, 5.0, 6.0]]]], dtype=np.float32),
-            # TODO: variadic (rdar://59559656)
-            # np.array([[1, 2, 3],
-            #          [4, 5, 6]], dtype=np.float32),
             np.array([[[1, 2, 3]], [[4, 5, 6]]], dtype=np.float32),
             np.array([[[1, 2, 3]], [[4, 5, 6]]], dtype=np.float32),
         ]
 
         input_values = {
             "x": np.array([[1, 2, 3], [4, 5, 6]], dtype=np.float32),
-            # TODO: variadic (rdar://59559656)
-            # "x2": np.array([[[1, 2, 3],[4, 5, 6]]], dtype=np.float32),
             "shape": np.array([2, 1, 3], dtype=np.float32),
             "shape2": np.array([2, 1, 3], dtype=np.float32),
         }
@@ -379,7 +538,7 @@ class TestReverse:
     def test_builder_eval(self):
         val = np.array([[-1.0, 7.0, -3.0], [4.0, -5.0, 8.0]], dtype=np.float32)
         res = mb.reverse(x=val, axes=[0])
-        assert is_close(np.flip(val, axis=0), res.val)
+        np.testing.assert_allclose(np.flip(val, axis=0), res.val, atol=1e-04, rtol=1e-05)
 
     @pytest.mark.parametrize(
         "use_cpu_only, backend", itertools.product([True, False], backends,)
@@ -571,9 +730,9 @@ class TestSliceBySize:
         v_1 = mb.slice_by_size(x=x, begin=(0, 1, 0), size=(-1, -1, -1))
         v_2 = mb.slice_by_size(x=x, begin=(0, 1, 0), size=(-1, -1, 3))
         v_3 = mb.slice_by_size(x=x, begin=(0, -2, 0), size=(-1, -1, 3))
-        assert is_close(x[:, 1:, :], v_1.val)
-        assert is_close(x[:, 1:, :3], v_2.val)
-        assert is_close(x[:, -2:, :3], v_3.val)
+        np.testing.assert_allclose(x[:, 1:, :], v_1.val, atol=1e-04, rtol=1e-05)
+        np.testing.assert_allclose(x[:, 1:, :3], v_2.val, atol=1e-04, rtol=1e-05)
+        np.testing.assert_allclose(x[:, -2:, :3], v_3.val, atol=1e-04, rtol=1e-05)
 
 
 class TestSpaceToDepth:
@@ -608,9 +767,9 @@ class TestSpaceToDepth:
 
 class TestSqueeze:
     @pytest.mark.parametrize(
-        "use_cpu_only, backend", itertools.product([True, False], backends,)
+        "use_cpu_for_conversion, backend", itertools.product([True, False], backends,)
     )
-    def test_builder_to_backend_smoke(self, use_cpu_only, backend):
+    def test_builder_to_backend_smoke(self, use_cpu_for_conversion, backend):
         x = np.array([[[[1], [2], [3]]]], dtype=np.float32)
         input_placeholders = {"x": mb.placeholder(shape=x.shape)}
 
@@ -644,7 +803,7 @@ class TestSqueeze:
             input_values,
             expected_output_types,
             expected_outputs,
-            use_cpu_only=use_cpu_only,
+            use_cpu_only=use_cpu_for_conversion,
             backend=backend,
         )
 
@@ -652,7 +811,15 @@ class TestSqueeze:
     def test_builder_eval(self):
         x = np.array([[[[1], [2], [3]], [[4], [5], [6]]]], dtype=np.float32)
         v = mb.squeeze(x=x, axes=(-4, 3))
-        assert is_close(np.squeeze(x, axis=(-4, 3)), v.val)
+        np.testing.assert_allclose(np.squeeze(x, axis=(-4, 3)), v.val, atol=1e-04, rtol=1e-05)
+
+    @ssa_fn
+    def test_builder_eval_rank_0(self):
+        x = np.array([1], dtype=np.float32)
+        v = mb.squeeze(x=x)
+        assert v.shape == ()
+        assert type(v.val) == np.float32
+        assert np.isclose(np.squeeze(x), v.val)
 
 
 class TestTranspose:
@@ -705,7 +872,7 @@ class TestTranspose:
     def test_builder_eval(self):
         x = np.array([[1, 2, 3], [4, 5, 6]], dtype=np.float32)
         v = mb.transpose(x=x, perm=(1, 0))
-        assert is_close(x.T, v.val)
+        np.testing.assert_allclose(x.T, v.val, atol=1e-04, rtol=1e-05)
 
     @pytest.mark.parametrize(
         "use_cpu_only, backend", itertools.product([True, False], backends,)
@@ -713,7 +880,6 @@ class TestTranspose:
     def test_builder_to_backend_symbolic(self, use_cpu_only, backend):
         s0 = get_new_symbol()
 
-        # Test variadic (rdar://59559656)
         input_placeholders = {
             "x": mb.placeholder(shape=(2, s0)),
         }
@@ -805,6 +971,74 @@ class TestPixelShuffle:
         )
 
 
+@pytest.mark.skipif(ct.utils._macos_version() < (13, 0), reason="New functionality in macOS13/iOS16")
+class TestPixelUnshuffle:
+    @pytest.mark.parametrize(
+        "use_cpu_only, backend", itertools.product([True, False], backends,)
+    )
+    def test_builder_to_backend_smoke(self, use_cpu_only, backend):
+        if backend[0] == "neuralnetwork":
+            pytest.skip("nn backend not supported")
+
+        val = np.array([[[[9.0, 5.0], [1.0, 3.0]]]], dtype=np.float32)
+        input_placeholders = {"x": mb.placeholder(shape=val.shape)}
+        input_values = {"x": val}
+
+        def build(x):
+            return [mb.pixel_unshuffle(x=x, downscale_factor=np.uint32(2))]
+
+        expected_output_types = (1, 4, 1, 1, types.fp32)
+        expected_outputs = np.array([[[[9.0]], [[5.0]], [[1.0]], [[3.0]]]], dtype=np.float32)
+
+        run_compare_builder(
+            build,
+            input_placeholders,
+            input_values,
+            expected_output_types,
+            expected_outputs,
+            use_cpu_only=use_cpu_only,
+            backend=backend,
+            minimum_deployment_target=ct.target.iOS16,
+        )
+
+    @pytest.mark.skipif(not testing_reqs._HAS_TORCH, reason="PyTorch not found.")
+    @pytest.mark.parametrize(
+        "use_cpu_only, backend, shape, downscale_factor",
+        itertools.product(
+            [True, False],
+            backends,
+            [(1, 2, 4, 4), (2, 1, 8, 4)],
+            [2, 4],
+        ),
+    )
+    def test_builder_to_backend_stress(
+        self, use_cpu_only, backend, shape, downscale_factor,
+    ):
+        if backend[0] == "neuralnetwork":
+            pytest.skip("nn backend not supported")
+            
+        val = np.random.rand(*shape)
+        input_placeholders = {"x": mb.placeholder(shape=val.shape)}
+        input_values = {"x": val}
+
+        def build(x):
+            return [mb.pixel_unshuffle(x=x, downscale_factor=np.uint32(downscale_factor))]
+
+        torch_pixel_unshuffle = torch.nn.PixelUnshuffle(downscale_factor)
+        expected_outputs = [torch_pixel_unshuffle(torch.Tensor(val)).numpy()]
+        expected_output_types = [o.shape[:] + (types.fp32,) for o in expected_outputs]
+        run_compare_builder(
+            build,
+            input_placeholders,
+            input_values,
+            expected_output_types,
+            expected_outputs,
+            use_cpu_only=use_cpu_only,
+            backend=backend,
+            minimum_deployment_target=ct.target.iOS16,
+        )
+
+
 class TestSlidingWindows:
     @pytest.mark.parametrize(
         "use_cpu_only, backend", itertools.product([True, False], backends,)
@@ -923,7 +1157,6 @@ class TestConcat:
     )
     def test_builder_to_backend_type_promotion(self, use_cpu_only, backend):
         t1 = np.array([[1, 2], [4, 5]], dtype=np.float32)
-        t2 = np.array([[7, 8]], dtype=np.float32)
 
         input_placeholders = {
             "x": mb.placeholder(shape=t1.shape),
@@ -963,7 +1196,6 @@ class TestConcat:
             [False, True],
         )
     )
-    @pytest.mark.skip(reason="rdar://65198011 (Re-enable Conv3dTranspose, concat interleave and DynamicTile unit tests)")
     def test_builder_to_backend_stress_interleave(self, use_cpu_only, backend,
                                                   rank, n_inputs, negative_index):
 
@@ -1059,7 +1291,7 @@ class TestConcat:
             np.random.rand(1, 1, 3, 2),
         ]
         v = mb.concat(values=values, axis=2)
-        assert is_close(np.concatenate(values, 2), v.val)
+        np.testing.assert_allclose(np.concatenate(values, 2), v.val, atol=1e-04, rtol=1e-05)
 
     @ssa_fn
     def test_builder_eval_failure(self):
@@ -1068,7 +1300,7 @@ class TestConcat:
             np.random.rand(1, 1, 3, 1),
         ]
         with pytest.raises(ValueError):
-            v = mb.concat(values=values, axis=2)
+            mb.concat(values=values, axis=2)
 
 
 class TestSplit:
@@ -1118,7 +1350,7 @@ class TestSplit:
         vs = mb.split(x=t, num_splits=3, axis=0)
         es = np.split(t, [1, 2, 3], axis=0)
         for v, e in zip(vs, es):
-            assert is_close(e, v.val)
+            np.testing.assert_allclose(e, v.val, atol=1e-04, rtol=1e-05)
 
 
 class TestStack:
@@ -1165,4 +1397,4 @@ class TestStack:
             np.random.rand(1, 1, 3, 2).astype(np.float32),
         ]
         v = mb.stack(values=values, axis=2)
-        assert is_close(np.stack(values, 2), v.val)
+        np.testing.assert_allclose(np.stack(values, 2), v.val, atol=1e-04, rtol=1e-05)
