@@ -31,6 +31,8 @@ from . import (
     noop_elimination,
     onehot_matmul_to_gather,
     pad_conv_connect,
+    prelu_fusion,
+    prelu_to_lrelu,
     quantization_passes,
     rank0_expand_dims_swap,
     reduce_mean_fusion,
@@ -40,11 +42,11 @@ from . import (
     replace_stack_reshape,
     sanitize_input_output_names,
     topological_reorder,
-    use_reflection_padding
+    use_reflection_padding,
+    update_output_dtypes,
 )
 
 from coremltools.converters.mil.experimental.passes import (
-    generic_gelu_tanh_approximation_fusion,
     generic_layernorm_instancenorm_pattern_fusion,
     generic_linear_bias_fusion,
     generic_conv_batchnorm_fusion,
