@@ -3,12 +3,13 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from coremltools.converters.mil.mil.passes.quantization_passes import AbstractQuantizationPass
-from coremltools.converters.mil.mil.passes.pass_registry import PASS_REGISTRY
 import logging as _logging
-from coremltools.converters._profile_utils import _profile
 from tqdm import tqdm as _tqdm
-from coremltools.converters.mil.experimental.passes.generic_pass_infrastructure import  PassContainer
+
+from coremltools.converters._profile_utils import _profile
+from coremltools.converters.mil.experimental.passes.generic_pass_infrastructure import PassContainer
+from coremltools.converters.mil.mil.passes.pass_registry import PASS_REGISTRY
+from coremltools.converters.mil.mil.passes.quantization_passes import AbstractQuantizationPass
 
 
 @_profile
@@ -35,6 +36,7 @@ def apply_common_pass_pipeline(prog, passes):
         return
 
     common_passes = [
+        "common::update_output_dtypes",
         "common::cast_optimization",
         "common::const_elimination",
         "common::sanitize_input_output_names",
@@ -43,7 +45,7 @@ def apply_common_pass_pipeline(prog, passes):
         "common::const_elimination",
         "common::loop_invariant_elimination",
         "common::remove_symbolic_reshape",
-        'common::noop_elimination',
+        "common::noop_elimination",
         "common::fuse_matmul_weight_bias",
         "common::fuse_linear_bias",
         "common::fuse_gelu_tanh_approximation",
@@ -53,7 +55,7 @@ def apply_common_pass_pipeline(prog, passes):
         "common::use_reflection_padding",
         "common::merge_consecutive_paddings", # Should come after use_reflection_padding, which will introduce new padding layers
         "common::pad_conv_connect", # Should come after merge_consecutive_paddings
-        'common::image_input_preprocess',
+        "common::image_input_preprocess",
         "common::replace_stack_reshape", # should come before detect_concat_interleave since it may add concat
         "common::reduce_transposes",
         "common::fuse_conv_scale",
@@ -65,9 +67,12 @@ def apply_common_pass_pipeline(prog, passes):
         "common::fuse_conv_batchnorm", # should come after fuse_elementwise_to_batchnorm
         "common::fuse_conv_scale", # Re-run the fuse conv scale pass after the conv and batch_norm are fused
         "common::fuse_conv_bias", # Re-run the fuse conv bias pass after the conv and batch_norm are fused
+        "common::fuse_conv_batchnorm", # In some cases, we need to run conv / batch_norm fusion again after the fuse_conv_scale and fuse_conv_bias passes
         "common::detect_concat_interleave",
         "common::concat_to_pixel_shuffle", # should come after detect_concat_interleave and after replace_stack_reshape
-        # "remove_redundant_ops" pass should be applied towards the end, once other graph passes have done their optimizations.
+        "common::fuse_prelu", # reduce_transpose pass should run before and after this pass (the one after will be run during the cleanup passes stage)
+        "common::prelu_to_lrelu",
+        #  "remove_redundant_ops" pass should be applied towards the end, once other graph passes have done their optimizations.
         # For instance, it should come after passes such as "reduce_transpose" that can introduce redundant transposes
         # in the network (while reducing the total number of transposes), and after passes such as "fuse_layernorm_or_instancenorm"
         # which detects patterns that involve redundant ops ("sub") etc.
@@ -82,6 +87,8 @@ def apply_common_pass_pipeline(prog, passes):
             _apply([p], type(p).__name__)
 
     cleanup_passes = [
+        "common::dead_code_elimination",
+        "common::const_elimination",
         "common::cast_optimization",
         "common::const_elimination",
         "common::loop_invariant_elimination",
