@@ -3,12 +3,14 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-
-from coremltools.converters.mil.mil.passes.pass_registry import register_pass
-from coremltools.converters.mil.mil.passes.graph_pass import AbstractGraphPass
 from coremltools.converters.mil.mil import Builder as mb
-from .helper import _check_child_op_type, _check_var_scalar_value
-import numpy as np
+from coremltools.converters.mil.mil.passes.graph_pass import AbstractGraphPass
+from coremltools.converters.mil.mil.passes.helper import (
+    _check_child_op_type,
+    _check_var_scalar_value,
+    block_context_manager,
+)
+from coremltools.converters.mil.mil.passes.pass_registry import register_pass
 
 def _try_to_transform(op, block):
     ops_to_remove = []
@@ -66,7 +68,16 @@ def _try_to_transform(op, block):
         ops_to_remove.append(mul_op_2)
 
     else:
-        return False
+        other_parent_op = mul_op.x.op if mul_op.y == add_op.outputs[0] else mul_op.y.op
+        if other_parent_op.op_type != "mul":
+            return False
+        if not (_check_var_scalar_value(other_parent_op.x, 0.5) or _check_var_scalar_value(other_parent_op.y, 0.5)):
+            return False
+        if not (other_parent_op.x == root_var or other_parent_op.y == root_var):
+            return False
+        ops_to_remove.append(other_parent_op)
+        ops_to_remove.append(mul_op)
+        mul_op_2 = mul_op
 
     # check that none of the op in this pattern is connected to the output
     # (except the last mul op)
@@ -86,7 +97,7 @@ def _try_to_transform(op, block):
     block.remove_ops(ops_to_remove)
     return True
 
-
+@block_context_manager
 def _fuse_gelu_exact_block(block):
     fusion_occurred = False
     for op in list(block.operations):
@@ -99,11 +110,10 @@ def _fuse_gelu_exact_block(block):
             continue
 
         if op.op_type in ["mul", "real_div"]:
-            with block:
-                fusion_occurred = _try_to_transform(op, block)
-                # has to break as the downstream iterator is affected.
-                if fusion_occurred:
-                    return fusion_occurred
+            fusion_occurred = _try_to_transform(op, block)
+            # has to break as the downstream iterator is affected.
+            if fusion_occurred:
+                return fusion_occurred
     return fusion_occurred
 
 @register_pass(namespace="common")
@@ -126,7 +136,13 @@ class fuse_gelu_exact(AbstractGraphPass):
           |                                                   |
           |----------------------------------------------------
 
-    both result in :
+    (3)
+        [...] ----> div (1.414) ---> erf ---> add (1) -----> mul ------> [...]
+          |                                                   ^
+          |                                                   |
+          |---------------> mul(0.5) --------------------------
+
+    all result in :
         [...] ----> gelu (mode=EXACT) ---> [...]
     """
     def apply(self, prog):
