@@ -3,12 +3,17 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from coremltools.converters.mil.mil import Builder as mb
-from coremltools.converters.mil.mil.passes.pass_registry import register_pass
-from coremltools.converters.mil.mil.passes.graph_pass import AbstractGraphPass
-from .helper import _check_child_op_type
 import numpy as np
 
+from coremltools.converters.mil.mil import Builder as mb
+from coremltools.converters.mil.mil.passes.graph_pass import AbstractGraphPass
+from coremltools.converters.mil.mil.passes.helper import (
+    _check_child_op_type,
+    block_context_manager,
+)
+from coremltools.converters.mil.mil.passes.pass_registry import register_pass
+
+
 def _match_pattern(block, padding_op):
 
     if padding_op.op_type != "pad":
@@ -37,7 +42,7 @@ def _match_pattern(block, padding_op):
             return False
     else:
         # if the padding is not constant, then we can't merge if both pads affected the same side of the image
-        if any(i != 0 and j != 0 for (i,j) in zip(first_pad, child_pad)):
+        if any(i != 0 and j != 0 for (i, j) in zip(first_pad, child_pad)):
             return False
 
     return _replace_ops(block, padding_op, child_padding_op, final_pad)
@@ -45,20 +50,18 @@ def _match_pattern(block, padding_op):
 
 def _replace_ops(block, padding_op, child_padding_op, final_pad):
 
-    with block:
+    mode = padding_op.inputs["mode"].val
+    x = mb.pad(x=padding_op.inputs["x"], pad=final_pad, mode=mode, constant_val=padding_op.inputs["constant_val"].val,
+               before_op=padding_op)
+    padding_op.enclosing_block.replace_uses_of_var_after_op(
+        anchor_op=padding_op, old_var=child_padding_op.outputs[0], new_var=x
+    )
 
-        mode = padding_op.inputs["mode"].val
-        x = mb.pad(x=padding_op.inputs["x"], pad=final_pad, mode=mode, constant_val=padding_op.inputs["constant_val"].val,
-                   before_op=padding_op)
-        padding_op.enclosing_block.replace_uses_of_var_after_op(
-            anchor_op=padding_op, old_var=child_padding_op.outputs[0], new_var=x
-        )
-
-        block.remove_ops([padding_op, child_padding_op])
+    block.remove_ops([padding_op, child_padding_op])
 
     return True
 
-
+@block_context_manager
 def _merge_padding_block(block):
     for op in list(block.operations):
         result = _match_pattern(block, op)
@@ -66,6 +69,7 @@ def _merge_padding_block(block):
             return True
     return False
 
+
 @register_pass(namespace="common")
 class merge_consecutive_paddings(AbstractGraphPass):
     """
