@@ -4,19 +4,25 @@
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
 import itertools
+
 import numpy as np
 import pytest
 
 from coremltools.converters.mil.mil import Builder as mb
-from coremltools.converters.mil.mil.passes.pass_registry import PASS_REGISTRY
 from coremltools.converters.mil.testing_utils import (
+    apply_pass_and_basic_check,
     assert_model_is_valid,
     get_op_types_in_program,
-    apply_pass_and_basic_check,
 )
 
 
-@pytest.mark.parametrize("op_type, pos, val", itertools.product(['add', 'mul', 'floor_div', 'pow', 'real_div', 'sub'], ['x', 'y'], [0, 1, [0, 0, 0, 0], [1, 1, 1, 1]]))
+@pytest.mark.parametrize(
+    "op_type, pos, val", itertools.product(
+        ['add', 'mul', 'floor_div', 'pow', 'real_div', 'sub'],
+        ['x', 'y'],
+        [0., 1., [0., 0., 0., 0.], [1., 1., 1., 1.]]
+    )
+)
 def test_elementwise_elimination(op_type, pos, val):
     if 'div' in op_type and np.prod(val) == 0:
         return
@@ -39,25 +45,21 @@ def test_elementwise_elimination(op_type, pos, val):
     original_program = [op_type, "relu"]
     new_program = original_program
     if op_type in {'add'}:
-        if val == 0 or val == [0, 0, 0, 0]:
+        if val == 0. or val == [0., 0., 0., 0.]:
             new_program = ["relu"]
     elif op_type in {'mul'}:
-        if val == 1 or val == [1, 1, 1, 1]:
+        if val == 1. or val == [1., 1., 1., 1.]:
             new_program = ["relu"]
     elif op_type in {'real_div'}:
-        # TODO(rdar://79925291): Remove this branch and add `real_div` to the
-        # following elif once fp32 casts for `real_div` are no longer required.
-        original_program = ["cast"] + original_program
-        new_program = original_program
-        if pos == 'y' and (val == 1 or val == [1, 1, 1, 1]):
-            new_program = ["cast", "relu"]
+        if pos == 'y' and (val == 1. or val == [1., 1., 1., 1.]):
+            new_program = ["relu"]
     elif op_type in {'pow', 'floor_div'}:
-        if pos == 'y' and (val == 1 or val == [1, 1, 1, 1]):
+        if pos == 'y' and (val == 1. or val == [1., 1., 1., 1.]):
             new_program = ["relu"]
     elif op_type in {'sub'}:
-        if pos == 'y' and (val == 0 or val == [0, 0, 0, 0]):
+        if pos == 'y' and (val == 0. or val == [0., 0., 0., 0.]):
             new_program = ["relu"]
-            
+
     assert get_op_types_in_program(prev_prog) == original_program
     assert get_op_types_in_program(prog) == new_program
     assert_model_is_valid(
@@ -70,7 +72,7 @@ def test_elementwise_broadcast():
 
     @mb.program(input_specs=[mb.TensorSpec(shape=[4])])
     def prog(x):
-        r1 = mb.add(x=x, y=[[0, 0, 0, 0], [0, 0, 0, 0]])
+        r1 = mb.add(x=x, y=[[0., 0., 0., 0.], [0., 0., 0., 0.]])
         return mb.relu(x=r1)
 
     prev_prog, prev_block, block = apply_pass_and_basic_check(
@@ -90,7 +92,7 @@ def test_reshape_elimination():
     @mb.program(input_specs=[mb.TensorSpec(shape=(2, 4))])
     def prog(x):
         r1 = mb.reshape(x=x, shape=[1, 8])
-        r2 = mb.reshape(x=r1, shape=[1, 8])
+        mb.reshape(x=r1, shape=[1, 8])
         return mb.relu(x=r1)
 
     prev_prog, prev_block, block = apply_pass_and_basic_check(
@@ -194,6 +196,30 @@ def test_slicebyindex_full_elimination():
         expected_output_shapes={block.outputs[0].name: (2, 4)},
     )
 
+def test_slicebyindex_negative_stride():
+    @mb.program(input_specs=[mb.TensorSpec(shape=(2, 4))])
+    def prog(x):
+        r1 = mb.slice_by_index(
+            x=x, 
+            begin=[0, 0],
+            end=[0, 0],
+            stride=[1, -1],
+            begin_mask=[True, True], 
+            end_mask=[True, True]
+        ) 
+        return mb.relu(x=r1)
+
+    prev_prog, prev_block, block = apply_pass_and_basic_check(
+        prog, "common::noop_elimination"
+    )
+    assert get_op_types_in_program(prev_prog) == ["slice_by_index", "relu"]
+    assert get_op_types_in_program(prog) == ["slice_by_index", "relu"]
+    assert_model_is_valid(
+        prog,
+        {"x": (2, 4)},
+        expected_output_shapes={block.outputs[0].name: (2, 4)},
+    )
+
 
 @pytest.mark.parametrize("begin_mask, end_mask",
                          itertools.product(itertools.product([True, False],[True, False]),
@@ -401,5 +427,3 @@ def test_transpose_elimination():
         {"x": (2, 3, 4)},
         expected_output_shapes={block.outputs[0].name: (2, 3, 4)},
     )
-
-
