@@ -3,24 +3,21 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from coremltools._deps import _IS_MACOS
+import numpy as np
+
 from coremltools.converters.mil.mil import Builder as mb
 from coremltools.converters.mil.testing_utils import (
     assert_model_is_valid,
     get_op_types_in_program,
     apply_pass_and_basic_check,
 )
-from coremltools.converters.mil.testing_reqs import ct
 
-import pytest
-
-import numpy as np
 
 np.random.seed(1984)
 
 
 class TestUseReflectionPadding:
-    
+
     def test_success_w_axis(self):
         @mb.program(input_specs=[mb.TensorSpec(shape=(1, 2, 6, 8))])
         def prog(x1):
@@ -43,6 +40,7 @@ class TestUseReflectionPadding:
             expected_output_shapes={block.outputs[0].name: (1, 2, 6, 10)},
         )
 
+
     def test_success_w_axis_multiple(self):
         @mb.program(input_specs=[mb.TensorSpec(shape=(1, 2, 6, 8))])
         def prog(x1):
@@ -67,6 +65,7 @@ class TestUseReflectionPadding:
             expected_output_shapes={block.outputs[0].name: (1, 2, 6, 12)},
         )
 
+
     def test_success_h_axis(self):
         @mb.program(input_specs=[mb.TensorSpec(shape=(1, 2, 6, 8))])
         def prog(x1):
@@ -87,14 +86,15 @@ class TestUseReflectionPadding:
             prog,
             inputs,
             expected_output_shapes={block.outputs[0].name: (1, 2, 8, 8)},
-        )        
+        )
+
 
     def test_failure_wrong_concat_order(self):
         @mb.program(input_specs=[mb.TensorSpec(shape=(1, 2, 6, 8))])
         def prog(x1):
             left = mb.slice_by_index(x=x1, begin=[0, 0, 1, 0], end=[0, 0, 2, 0], end_mask=[True, True, False, True])
             right = mb.slice_by_index(x=x1, begin=[0, 0, -2, 0], end=[0, 0, -1, 0], end_mask=[True, True, False, True])
-            # Concat is not in correct order 
+            # Concat is not in correct order
             x = mb.concat(values=[left, right, x1], axis=2)
 
             return x
@@ -110,7 +110,8 @@ class TestUseReflectionPadding:
             prog,
             inputs,
             expected_output_shapes={block.outputs[0].name: (1, 2, 8, 8)},
-        )        
+        )
+
 
     def test_failure_wrong_concat_order_2(self):
         @mb.program(input_specs=[mb.TensorSpec(shape=(1, 2, 6, 8))])
@@ -137,6 +138,7 @@ class TestUseReflectionPadding:
             expected_output_shapes={block.outputs[0].name: (1, 2, 6, 12)},
         )
 
+
     def test_failure_wrong_slice_size(self):
         @mb.program(input_specs=[mb.TensorSpec(shape=(1, 2, 6, 8))])
         def prog(x1):
@@ -158,7 +160,8 @@ class TestUseReflectionPadding:
             prog,
             inputs,
             expected_output_shapes={block.outputs[0].name: (1, 2, 9, 8)},
-        )        
+        )
+
 
     def test_failure_not_all_same_input(self):
         @mb.program(input_specs=[mb.TensorSpec(shape=(1, 2, 6, 8)), mb.TensorSpec(shape=(1, 2, 6, 8))])
@@ -185,6 +188,7 @@ class TestUseReflectionPadding:
             expected_output_shapes={block.outputs[0].name: (1, 2, 6, 12)},
         )
 
+
     def test_failure_slice_output(self):
         @mb.program(input_specs=[mb.TensorSpec(shape=(1, 2, 6, 8))])
         def prog(x1):
@@ -207,3 +211,22 @@ class TestUseReflectionPadding:
             inputs,
             expected_output_shapes={block.outputs[0].name: (1, 2, 6, 10), block.outputs[1].name: (1, 2, 6, 1)},
         )
+
+
+    def test_concat_input_only(self):
+        @mb.program(input_specs=[mb.TensorSpec(shape=(1, 2, 6, 8))])
+        def prog(x):
+            x = mb.concat(values=[x, x, x], axis=0)
+            return x
+
+        prev_prog, _, block = apply_pass_and_basic_check(
+            prog, "common::use_reflection_padding"
+        )
+        assert get_op_types_in_program(prog) == ["concat"]
+
+        inputs = {"x": (1, 2, 6, 8)}
+        assert_model_is_valid(
+            prog,
+            inputs,
+            expected_output_shapes={block.outputs[0].name: (3, 2, 6, 8)},
+        )
