@@ -5,6 +5,7 @@
 
 import copy
 from io import BytesIO
+import itertools
 import numpy as np
 import os
 from os import getcwd, chdir
@@ -20,6 +21,7 @@ from coremltools.converters.mil import Builder as mb
 from coremltools.converters.mil.frontend.torch.test.testing_utils import _copy_input_data
 from coremltools.converters.mil.mil import get_new_symbol, Program, Function
 from coremltools.converters.mil.testing_utils import get_op_types_in_program
+from coremltools.proto import FeatureTypes_pb2 as ft
 
 from coremltools._deps import (
     _HAS_TF_1,
@@ -61,12 +63,12 @@ class TestTensorFlow1ConverterExamples:
             x = tf.placeholder(tf.float32, shape=(1, 2, 3), name="input")
             y = tf.nn.relu(x, name="output")
 
-        mlmodel = ct.convert(graph)
+        mlmodel = ct.convert(graph, compute_units=ct.ComputeUnit.CPU_ONLY)
 
         test_input = np.random.rand(1, 2, 3) - 0.5
         with tf.compat.v1.Session(graph=graph) as sess:
             expected_val = sess.run(y, feed_dict={x: test_input})
-        results = mlmodel.predict({"input": test_input}, useCPUOnly=True)
+        results = mlmodel.predict({"input": test_input})
         np.testing.assert_allclose(results["output"], expected_val)
 
     @staticmethod
@@ -114,9 +116,9 @@ class TestTensorFlow1ConverterExamples:
         # (3) Not specify inputs at all. `inputs` is optional for TF. When
         # inputs is not specified, convert() infers inputs from Placeholder
         # nodes.
-        mlmodel = ct.convert(pb_path, outputs=["output"])
+        mlmodel = ct.convert(pb_path, outputs=["output"], compute_units=ct.ComputeUnit.CPU_ONLY)
 
-        results = mlmodel.predict({"input": test_input}, useCPUOnly=True)
+        results = mlmodel.predict({"input": test_input})
         np.testing.assert_allclose(results["output"], expected_val)
         mlmodel_path = os.path.join(save_path, "model.mlmodel")
         # Save the converted model
@@ -145,12 +147,12 @@ class TestTensorFlow1ConverterExamples:
 
         # SavedModel directory generated by TensorFlow 1.x
         # when converting from SavedModel dir, inputs / outputs are optional
-        mlmodel = ct.convert(save_path)
+        mlmodel = ct.convert(save_path, compute_units=ct.ComputeUnit.CPU_ONLY)
 
         # Need input output names to call mlmodel
         # x.name == 'Placeholder:0'. Strip out ':0'
         input_name = x.name.split(":")[0]
-        results = mlmodel.predict({input_name: test_input}, useCPUOnly=True)
+        results = mlmodel.predict({input_name: test_input})
         # y.name == 'Relu:0'. output_name == 'Relu'
         output_name = y.name.split(":")[0]
         np.testing.assert_allclose(results[output_name], expected_val)
@@ -470,11 +472,8 @@ class TestPyTorchConverterExamples:
         """
         if ct.utils._is_macos():
             results = mlmodel.predict({"input": example_input.numpy()})
-            expected = model(example_input)
-            np.testing.assert_allclose(
-                list(results.values())[0], expected.detach().numpy(),
-                atol=1e-8, rtol=1e-2)
-
+            assert isinstance(results, dict)
+            
     @staticmethod
     def test_int64_inputs():
 
@@ -535,8 +534,8 @@ class TestPyTorchConverterExamples:
                 ],
             )
 
-        # Outputs must not be specified for PyTorch
-        with pytest.raises(ValueError, match=r"outputs must not be specified"):
+        # Outputs must be of type ct.ImageType or ct.TensorType
+        with pytest.raises(ValueError, match=r"must be a list of type ct.TensorType or ct.ImageType"):
             mlmodel = ct.convert(
                 traced_model,
                 inputs=[
@@ -600,7 +599,7 @@ class TestMILExamples:
     @staticmethod
     def test_tutorial():
         @mb.program(
-            input_specs=[mb.TensorSpec(shape=(1, 100, 100, 3)),]
+            input_specs=[mb.TensorSpec(shape=(1, 100, 100, 3))]
         )
         def prog(x):
             x = mb.relu(x=x, name="relu")
@@ -618,10 +617,11 @@ class TestMILExamples:
         # running predict() is only supported on macOS
         if ct.utils._is_macos():
             prediction = mlmodel.predict(
-                {"x": np.random.rand(1, 100, 100, 3).astype(np.float32),}
+                {"x": np.random.rand(1, 100, 100, 3).astype(np.float32)}
             )
             assert len(prediction) == 1
 
+
 class TestInvalidInput:
     @staticmethod
     def test_rank0_inputs_mil():
@@ -723,13 +723,56 @@ class TestInputs:
 
         mlmodel = ct.convert(prog)
 
-        expected_err_str = "Provided key \"x/0\", in the input dict, " \
-                           "does not match to any of the model input name\(s\), which are: .*"
-        with pytest.raises(KeyError, match=expected_err_str):
-            prediction = mlmodel.predict(
+        with pytest.raises(KeyError) as error_info:
+            mlmodel.predict(
                 {"x/0": np.random.rand(2, 3).astype(np.float32),
                  "y": np.random.rand(2, 3).astype(np.float32)}
             )
+        error_str = str(error_info.value)
+        assert "does not match any of the model input" in error_str
+
+    @staticmethod
+    def _test_variant_input_type_prediction(to_tensor):
+        prog = Program()
+        func_inputs = {"x": mb.placeholder(shape=[2, 3]),
+                       "y": mb.placeholder(shape=[2, 3])}
+        with Function(func_inputs) as ssa_fun:
+            x, y = ssa_fun.inputs["x"], ssa_fun.inputs["y"]
+            x = mb.relu(x=x, name="relu")
+            z = mb.add(x=x, y=y, name="out")
+            ssa_fun.set_outputs([z])
+        prog.add_function("main", ssa_fun)
+
+        mlmodel = ct.convert(prog)
+        x_numpy = np.random.rand(2, 3)
+        y_numpy = np.random.rand(2, 3)
+        out_by_numpy = mlmodel.predict(
+            {"x": x_numpy,
+             "y": y_numpy}
+        )
+        out_by_tensor = mlmodel.predict(
+            {"x": to_tensor(x_numpy),
+             "y": to_tensor(y_numpy)}
+        )
+        np.allclose(out_by_numpy["out"], out_by_tensor["out"])
+
+    @staticmethod
+    @pytest.mark.skipif(not ct.utils._is_macos(), reason="test needs predictions")
+    def test_list_predict_input():
+        TestInputs._test_variant_input_type_prediction(lambda x: x.tolist())
+
+    @staticmethod
+    @pytest.mark.skipif(not _HAS_TF_1, reason="requires TensorFlow")
+    @pytest.mark.skipif(not ct.utils._is_macos(), reason="test needs predictions")
+    def test_tf_predict_input():
+        TestInputs._test_variant_input_type_prediction(tf.convert_to_tensor)
+
+    @staticmethod
+    @pytest.mark.skipif(not _HAS_TORCH, reason="requires Torch")
+    @pytest.mark.skipif(not ct.utils._is_macos(), reason="test needs predictions")
+    def test_torch_predict_input():
+        TestInputs._test_variant_input_type_prediction(torch.tensor)
+
 
 class TestFlexibleShape:
     @staticmethod
@@ -1124,12 +1167,12 @@ class TestFlexibleShape:
         mlmodel = ct.convert(
             traced_model,
             inputs=[tensor_input],
+            compute_units=ct.ComputeUnit.CPU_ONLY
         )
 
         if ct.utils._is_macos():
             result = mlmodel.predict(
                 {"input": example_input.detach().numpy().astype(np.float32)},
-                useCPUOnly=True,
             )
 
             # Verify outputs
@@ -1141,15 +1184,13 @@ class TestFlexibleShape:
             # Test (1, 3, 56, 56) shape (can't verify numerical parity with Torch
             # which doesn't support enumerated shape)
             test_input_x = np.random.rand(*shapes[1]).astype(np.float32)
-            results = mlmodel.predict({
-                "input": test_input_x})
+            mlmodel.predict({"input": test_input_x})
 
             # Test with a wrong shape
             with pytest.raises(RuntimeError,
                     match=r"not compatible with the model\'s feature"):
                 test_input_x = np.random.rand(1, 3, 29, 29).astype(np.float32)
-                results = mlmodel.predict({
-                    "input": test_input_x})
+                mlmodel.predict({"input": test_input_x})
 
     @staticmethod
     @pytest.mark.skipif(not _HAS_TF_2, reason=MSG_TF2_NOT_FOUND)
@@ -1184,6 +1225,30 @@ class TestFlexibleShape:
         spec = model.get_spec()
         assert len(spec.description.input[0].type.imageType.enumeratedSizes.sizes) == 2
 
+    @staticmethod
+    @pytest.mark.skipif(ct.utils._macos_version() < (13, 0), reason='Bug fixed in macOS13/iOS16')
+    @pytest.mark.skipif(not _HAS_TORCH, reason=MSG_TORCH_NOT_FOUND)
+    def test_mean_op():
+        # test for bug reported in https://github.com/apple/coremltools/issues/1420
+        class Network(torch.nn.Module):
+            def forward(self, x):
+                return torch.mean(x, dim=(2, 3), keepdim=True)
+
+        model = Network()
+        x = torch.rand(1, 3, 256, 256)
+        traced_model = torch.jit.trace(model, x)
+        input_x = ct.TensorType(shape=(1, 3, ct.RangeDim(default=256), ct.RangeDim(default=256)),
+                                name="input")
+        cml = ct.convert(traced_model,
+                         inputs=[input_x],
+                         outputs=[ct.TensorType(name="out")],
+                         convert_to="mlprogram",
+                         compute_units=ct.ComputeUnit.CPU_ONLY)
+        input_dict = {"input": np.random.rand(1, 3, 112, 112)}
+        if ct.utils._is_macos():
+            out = cml.predict(input_dict)["out"]
+            assert out.shape == (1, 3, 1, 1)
+
 
 class TestOptionalInput:
     @staticmethod
@@ -1269,8 +1334,9 @@ class TestOptionalInput:
                 assert isinstance(coreml_out["classLabel"], key_type)
 
         for class_type in ("str", "int"):
-            for backend in ("mlprogram", "neuralnetwork"):
-                _test_classifier(traced_model, example_input, class_type, backend)
+            _test_classifier(traced_model, example_input, class_type, "neuralnetwork")
+            if ct.utils._macos_version() >= (12, 0):
+                _test_classifier(traced_model, example_input, class_type, "mlprogram")
 
     @staticmethod
     @pytest.mark.skipif(not _HAS_TORCH, reason=MSG_TORCH_NOT_FOUND)
@@ -1303,6 +1369,9 @@ class TestOptionalInput:
             default_value=default_value)
 
         for compute_units in ct.ComputeUnit:
+            if compute_units == ct.ComputeUnit.CPU_AND_NE and ct.utils._macos_version() < (13, 0):
+                continue
+
             mlmodel = ct.convert(
                 traced_model,
                 inputs=[required_input, optional_input],
@@ -1577,23 +1646,47 @@ class TestMLProgramConverterExamples:
         mlmodel = ct.convert(prog, convert_to="mlprogram", compute_precision=ct.precision.FLOAT32)
         prog2 = mlmodel._get_mil_internal() # this will invoke a deepcopy on the prog
 
+
     @staticmethod
     @pytest.mark.skipif(not _HAS_TORCH or ct.utils._macos_version() < (12, 0),
                         reason=MSG_TORCH_NOT_FOUND)
-    def test_classifier():
+    @pytest.mark.parametrize(
+        "convert_to, provide_prob_output_argument",
+        itertools.product(
+            ["neuralnetwork", "mlprogram"],
+            [False, True],
+        )
+    )
+    def test_classifier(convert_to, provide_prob_output_argument):
         torch_model = torch.nn.ReLU().eval()
         traced_model = torch.jit.trace(torch_model, torch.rand(3,))
+        variable_name = "var_2"
+        class_label_name = "class_label"
+        classifier_config = ct.ClassifierConfig(
+                            class_labels=['a', 'b', 'c'],
+                            predicted_feature_name=class_label_name,
+                            predicted_probabilities_output=variable_name if provide_prob_output_argument else None,
+                            )
+
         model = ct.convert(
             traced_model,
             inputs=[ct.TensorType(shape=(3,))],
-            classifier_config = ct.ClassifierConfig(['a', 'b', 'c']),
-            convert_to='mlprogram'
+            classifier_config = classifier_config,
+            convert_to=convert_to,
         )
         spec = model.get_spec()
         input_name = spec.description.input[0].name
         out_dict = model.predict({input_name : np.array([1.0, 2.0, 3.0])})
-        assert 'classLabel' in out_dict
-        assert out_dict['classLabel'] == 'c'
+
+        assert class_label_name in out_dict
+        assert out_dict[class_label_name] == 'c'
+        if convert_to == "neuralnetwork":
+            assert variable_name in out_dict
+            assert isinstance(out_dict[variable_name], dict)
+        else:
+            output_dict_feature_name = class_label_name + "_probs"
+            assert output_dict_feature_name in out_dict
+            assert isinstance(out_dict[output_dict_feature_name], dict)
 
     @pytest.mark.skipif(not ct.utils._is_macos(), reason="Platform is not Mac OS")
     @pytest.mark.parametrize("skip_model_load", [True, False])
