@@ -2,9 +2,11 @@
 #
 # Use of this source code is governed by a BSD-3-clause license that can be
 # found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
-import coremltools as ct
+
 import pytest
 
+import coremltools as ct
+
 
 def _get_visible_items(d):
     return [x for x in dir(d) if not x.startswith("_")]
@@ -17,11 +19,7 @@ def _check_visible_modules(actual, expected):
     )
 
 
-class TestApiVisibilities:
-    """Test public coremltools API visibilities."""
-
-    def test_top_level(self):
-        expected = [
+EXPECTED_MODULES = [
             "ClassifierConfig",
             "ComputeUnit",
             "EnumeratedShapes",
@@ -30,6 +28,8 @@ class TestApiVisibilities:
             "SPECIFICATION_VERSION",
             "Shape",
             "TensorType",
+            "colorlayout",
+            "compression_utils",
             "convert",
             "converters",
             "libcoremlpython",
@@ -44,9 +44,15 @@ class TestApiVisibilities:
             "libmodelpackage",
             "libmilstoragepython",
         ]
+
+
+class TestApiVisibilities:
+    """Test public coremltools API visibilities."""
+
+    def test_top_level(self):
         if not ct.utils._is_macos():
-             expected.remove("libcoremlpython")
-        _check_visible_modules(_get_visible_items(ct), expected)
+             EXPECTED_MODULES.remove("libcoremlpython")
+        _check_visible_modules(_get_visible_items(ct), EXPECTED_MODULES)
 
     def test_utils(self):
         expected = [
@@ -65,13 +71,14 @@ class TestApiVisibilities:
         expected = [
             "MLModel",
             "datatypes",
+            "feature_vectorizer",
+            "ml_program",
             "model",
+            "nearest_neighbors",
             "neural_network",
             "pipeline",
             "tree_ensemble",
             "utils",
-            "nearest_neighbors",
-            "feature_vectorizer",
         ]
         _check_visible_modules(_get_visible_items(ct.models), expected)
 
@@ -87,6 +94,7 @@ class TestApiVisibilities:
             "short_description",
             "user_defined_metadata",
             "version",
+            "weights_dir",
         ]
         _check_visible_modules(_get_visible_items(ct.models.MLModel), expected)
 
@@ -96,13 +104,10 @@ class TestApiVisibilities:
             "NeuralNetworkBuilder",
             "SgdParams",
             "builder",
-            "datatypes",
             "flexible_shape_utils",
             "optimization_utils",
             "printer",
             "quantization_utils",
-            "set_training_features",
-            "set_transform_interface_params",
             "spec_inspection_utils",
             "update_optimizer_utils",
             "utils",
@@ -140,43 +145,23 @@ class TestApiVisibilities:
     def test_converters(self):
         expected = [
             "ClassifierConfig",
+            "ColorLayout",
             "EnumeratedShapes",
             "ImageType",
             "RangeDim",
             "Shape",
             "TensorType",
             "convert",
-            "keras",
             "libsvm",
             "mil",
-            "onnx",
             "sklearn",
             "xgboost",
         ]
         _check_visible_modules(_get_visible_items(ct.converters), expected)
 
-    @pytest.mark.skipif(
-        ct.utils._python_version() >= (3, 8, 0),
-        reason="Keras isn't compatible with Python 3.8+.",
-    )
-    @pytest.mark.xfail(
-         condition=not ct.utils._is_macos(),
-         reason="rdar://65138103 (Keras converter not exposed on Linux)",
-         run=False,
-     )
-    def test_converters_keras(self):
-        _check_visible_modules(_get_visible_items(ct.converters.keras), ["convert"])
-
     def test_converters_libsvm(self):
         _check_visible_modules(_get_visible_items(ct.converters.libsvm), ["convert"])
 
-    @pytest.mark.skipif(
-        ct.utils._python_version() >= (3, 8, 0),
-        reason="ONNX isn't compatible with Python 3.8+.",
-    )
-    def test_converters_onnx(self):
-        _check_visible_modules(_get_visible_items(ct.converters.onnx), ["convert"])
-
     def test_converters_sklearn(self):
         _check_visible_modules(_get_visible_items(ct.converters.sklearn), ["convert"])
 
@@ -200,6 +185,17 @@ class TestApiVisibilities:
             _get_visible_items(ct.models.neural_network.quantization_utils), expected
         )
 
+    def test_compression_utils(self):
+        expected = [
+            "affine_quantize_weights",
+            "palettize_weights",
+            "sparsify_weights",
+            "decompress_weights",
+        ]
+        _check_visible_modules(
+            _get_visible_items(ct.compression_utils), expected
+        )
+
     def test_models_neural_network_flexible_shape_utils(self):
         expected = [
             "NeuralNetworkImageSize",
