@@ -8,6 +8,7 @@ import tempfile
 import json
 
 from coremltools._deps import _HAS_SKLEARN, _HAS_XGBOOST
+from coremltools.models.utils import _macos_version
 
 if _HAS_XGBOOST:
     import xgboost
@@ -19,11 +20,6 @@ if _HAS_SKLEARN:
     from sklearn.preprocessing import OneHotEncoder
     from coremltools.converters import sklearn as skl_converter
 
-# Skipping tests due to <rdar://problem/66545691> xgboost/test_boosted_trees_*.py failing on CI
-import pytest
-pytestmark = pytest.mark.skip("Skipping for now. See 66545691 for details.")
-#
-
 @unittest.skipIf(not _HAS_SKLEARN, "Missing scikit-learn. Skipping tests.")
 class GradientBoostingRegressorScikitTest(unittest.TestCase):
     """
@@ -42,6 +38,12 @@ class GradientBoostingRegressorScikitTest(unittest.TestCase):
         scikit_model = GradientBoostingRegressor(random_state=1)
         scikit_model.fit(scikit_data["data"], scikit_data["target"])
 
+        s = 0
+        for est in scikit_model.estimators_:
+            for e in est:
+                s = s + e.tree_.node_count
+        cls.scikit_model_node_count = s
+
         # Save the data and the model
         cls.scikit_data = scikit_data
         cls.scikit_model = scikit_model
@@ -76,7 +78,7 @@ class GradientBoostingRegressorScikitTest(unittest.TestCase):
             -1
         ].treeEnsembleRegressor.treeEnsemble
         self.assertIsNotNone(tr)
-        self.assertEqual(len(tr.nodes), 1426)
+        self.assertEqual(len(tr.nodes), self.scikit_model_node_count)
 
     def test_conversion_bad_inputs(self):
 
@@ -91,6 +93,7 @@ class GradientBoostingRegressorScikitTest(unittest.TestCase):
             spec = skl_converter.convert(model, "data", "out")
 
 
+@unittest.skipIf(_macos_version() >= (10, 16), "rdar://problem/84898245")
 @unittest.skipIf(not _HAS_SKLEARN, "Missing scikit-learn. Skipping tests.")
 @unittest.skipIf(not _HAS_XGBOOST, "Skipping, no xgboost")
 class BoostedTreeRegressorXGboostTest(unittest.TestCase):
